""" This module contains dictionaries that can be used to set a
matplotlib plotting style.
It is mostly here to allow a consistent plotting style in tutorials,
but can be used to prepare any matplotlib figure.

Using a matplotlib version > 1.4 you can do::

    >>> import matplotlib.pyplot as pl
    >>> from gala.mpl_style import mpl_style
    >>> pl.style.use(mpl_style) # doctest: +SKIP

for older versions of matplotlib the following works::

    >>> import matplotlib as mpl
    >>> from gala.mpl_style import mpl_style
    >>> mpl.rcParams.update(mpl_style) # doctest: +SKIP

"""

mpl_style = {

    # Lines
    'lines.linewidth': 1.7,
    'lines.antialiased': True,
    'lines.marker': '.',
    'lines.markersize': 5.,

    # Patches
    'patch.linewidth': 1.0,
    'patch.facecolor': '#348ABD',
    'patch.edgecolor': '#CCCCCC',
    'patch.antialiased': True,

    # images
    'image.cmap': 'gist_heat',
    'image.origin': 'upper',

    # Font
    'font.size': 14.0,
    'text.latex.preamble': r'\usepackage{amsmath}',
    'axes.unicode_minus': False,

    # Axes
    'axes.facecolor': '#FFFFFF',
    'axes.edgecolor': '#AAAAAA',
    'axes.linewidth': 1.0,
    'axes.grid': False,
    'axes.titlesize': 'x-large',
    'axes.labelsize': 'large',
    'axes.labelcolor': 'k',
    'axes.axisbelow': True,

    # Ticks
    'xtick.major.size': 8,
    'xtick.minor.size': 4,
    'xtick.major.pad': 6,
    'xtick.minor.pad': 6,
    'xtick.color': '#565656',
    'xtick.direction': 'in',
    'ytick.major.size': 8,
    'ytick.minor.size': 4,
    'ytick.major.pad': 6,
    'ytick.minor.pad': 6,
    'ytick.color': '#565656',
    'ytick.direction': 'in',
    'xtick.labelsize': 'medium',
    'ytick.labelsize': 'medium',

    # Legend
    'legend.fancybox': True,
    'legend.loc': 'best',

    # Figure
    'figure.figsize': [8, 6],
    'figure.facecolor': '1.0',
    'figure.edgecolor': '0.50',
    'figure.subplot.hspace': 0.5,

    # Other
    'savefig.dpi': 300,
}

try:
    from cycler import cycler
    mpl_style['axes.prop_cycle'] = cycler('color', ['#1A1A1A', # black
                                                    '#2166AC', # blue
                                                    '#006837', # green
                                                    '#B2182B', # red
                                                    '#762A83', # purple
                                                    '#E08214',
                                                    '#80CDC1',
                                                    '#C51B7D',
                                                    '#FEE08B'])
except ImportError:
    mpl_style['axes.color_cycle'] = ['#1A1A1A', # black
                                     '#2166AC', # blue
                                     '#006837', # green
                                     '#B2182B', # red
                                     '#762A83', # purple
                                     '#E08214',
                                     '#80CDC1',
                                     '#C51B7D',
                                     '#FEE08B']



# coding: utf-8

from __future__ import division, print_function

__author__ = "adrn <adrn@astro.columbia.edu>"

# Project
from .._coord import (_test_sat_rotation_matrix, _test_to_sat_coords_roundtrip,
                      _test_car_to_cyl_roundtrip, _test_cyl_to_car_roundtrip)


"""
Note:
    This is just a way to get pytest to call tests implemented in Cython!
    See _coord.pyx for the actual test functions.
"""

def test_sat_rotation_matrix():
    _test_sat_rotation_matrix()

def test_to_sat_coords_roundtrip():
    _test_to_sat_coords_roundtrip()

def test_car_to_cyl_roundtrip():
    _test_car_to_cyl_roundtrip()

def test_cyl_to_car_roundtrip():
    _test_cyl_to_car_roundtrip()



# coding: utf-8

from __future__ import division, print_function

__author__ = "adrn <adrn@astro.columbia.edu>"

# Standard library
from collections import OrderedDict
import warnings

# Third-party
import numpy as np
from astropy.constants import G
import astropy.units as u
from astropy.utils import isiterable, InheritDocstrings
from astropy.extern import six

# Project
from ..integrate import *
from ..util import ImmutableDict, atleast_2d
from ..units import UnitSystem, DimensionlessUnitSystem
from ..dynamics import CartesianOrbit, CartesianPhaseSpacePosition

__all__ = ["PotentialBase", "CompositePotential"]

class PotentialBase(object):
    """
    A baseclass for defining pure-Python gravitational potentials.

    Subclasses must define (at minimum) a method that evaluates
    the value (energy) of the potential at a given position ``q``
    and time ``t``: ``_value(q, t)``. For integration, the subclasses
    must also define a method to evaluate the gradient,
    ``_gradient(q,t)``. Optionally, they may also define methods
    to compute the density and hessian: ``_density()``, ``_hessian()``.
    """

    def _prefilter_pos(self, q):
        if hasattr(q, 'unit'):
            q = q.decompose(self.units).value

        q = np.ascontiguousarray(atleast_2d(q, insert_axis=1))
        return q

    def __init__(self, parameters, units=None):
        # make sure the units specified are a UnitSystem instance
        if units is not None and not isinstance(units, UnitSystem):
            units = UnitSystem(*units)

        elif units is None:
            units = DimensionlessUnitSystem()

        self.units = units

        # in case the user specified an ordered dict
        self.parameters = OrderedDict()
        for k,v in parameters.items():
            if hasattr(v, 'unit'):
                self.parameters[k] = v.decompose(self.units)
            else:
                self.parameters[k] = v*u.one

        try:
            self.G = G.decompose(self.units).value
        except u.UnitConversionError:
            self.G = 1. # TODO: this is a HACK and could lead to user confusion

    def _value(self, q, t=0.):
        raise NotImplementedError()

    def value(self, q, t=0.):
        """
        Compute the value of the potential at the given position(s).

        Parameters
        ----------
        q : `~astropy.units.Quantity`, array_like
            The position to compute the value of the potential. If the
            input position object has no units (i.e. is an `~numpy.ndarray`),
            it is assumed to be in the same unit system as the potential.

        Returns
        -------
        E : `~astropy.units.Quantity`
            The potential energy per unit mass or value of the potential.
            If the input position has shape ``q.shape``, the output energy
            will have shape ``q.shape[1:]``.
        """
        q = self._prefilter_pos(q)
        return self._value(q, t=t) * self.units['energy'] / self.units['mass']

    def _gradient(self, q, t=0.):
        raise NotImplementedError()

    def gradient(self, q, t=0.):
        """
        Compute the gradient of the potential at the given position(s).

        Parameters
        ----------
        q : `~astropy.units.Quantity`, array_like
            The position to compute the value of the potential. If the
            input position object has no units (i.e. is an `~numpy.ndarray`),
            it is assumed to be in the same unit system as the potential.

        Returns
        -------
        grad : `~astropy.units.Quantity`
            The gradient of the potential. Will have the same shape as
            the input position array, ``q``.
        """
        q = self._prefilter_pos(q)

        try:
            return self._gradient(q, t=t) * self.units['acceleration']
        except NotImplementedError:
            raise NotImplementedError("This potential has no specified gradient function.")

    def _density(self, q, t=0.):
        raise NotImplementedError()

    def density(self, q, t=0.):
        """
        Compute the density value at the given position(s).

        Parameters
        ----------
        q : `~astropy.units.Quantity`, array_like
            The position to compute the value of the potential. If the
            input position object has no units (i.e. is an `~numpy.ndarray`),
            it is assumed to be in the same unit system as the potential.

        Returns
        -------
        dens : `~astropy.units.Quantity`
            The potential energy or value of the potential. If the input
            position has shape ``q.shape``, the output energy will have
            shape ``q.shape[1:]``.
        """
        q = self._prefilter_pos(q)

        try:
            return self._density(q, t=t) * self.units['mass density']
        except NotImplementedError:
            raise NotImplementedError("This potential has no specified density function.")

    def _hessian(self, q, t=0.):
        raise NotImplementedError()

    def hessian(self, q, t=0.):
        """
        Compute the Hessian of the potential at the given position(s).

        Parameters
        ----------
        q : `~astropy.units.Quantity`, array_like
            The position to compute the value of the potential. If the
            input position object has no units (i.e. is an `~numpy.ndarray`),
            it is assumed to be in the same unit system as the potential.

        Returns
        -------
        hess : `~astropy.units.Quantity`
            TODO:
        """
        q = self._prefilter_pos(q)

        try:
            return self._hessian(q, t=t) * self.units['acceleration'] / self.units['length']
        except NotImplementedError:
            raise NotImplementedError("This potential has no specified hessian function.")

    # ========================================================================
    # Things that use the base methods
    #
    def acceleration(self, q, t=0.):
        """
        Compute the acceleration due to the potential at the given
        position(s).

        Parameters
        ----------
        q : array_like, numeric
            Position to compute the acceleration at.

        Returns
        -------
        acc : `~numpy.ndarray`
            The acceleration. Will have the same shape as the input
            position array, ``q``.
        """
        return -self.gradient(q, t=t)

    def mass_enclosed(self, q, t=0.):
        """
        Estimate the mass enclosed within the given position by assuming the potential
        is spherical.

        Parameters
        ----------
        x : array_like, numeric
            Position to estimate the enclossed mass.

        Returns
        -------
        menc : `~astropy.units.Quantity`
            The potential energy or value of the potential. If the input
            position has shape ``q.shape``, the output energy will have
            shape ``q.shape[1:]``.
        """
        q = self._prefilter_pos(q)

        # Fractional step-size in radius
        h = 0.01

        # Radius
        r = np.sqrt(np.sum(q**2, axis=0))

        epsilon = h*q/r[np.newaxis]

        dPhi_dr_plus = self.value(q + epsilon, t=t)
        dPhi_dr_minus = self.value(q - epsilon, t=t)
        diff = dPhi_dr_plus - dPhi_dr_minus

        if isinstance(self.units, DimensionlessUnitSystem):
            raise ValueError("No units specified when creating potential object.")
        Gee = G.decompose(self.units).value

        return np.abs(r*r * diff / Gee / (2.*h)) * self.units['mass']

    # ========================================================================
    # Python special methods
    #
    def __call__(self, q):
        return self.value(q)

    def __repr__(self):
        pars = ""
        if not isinstance(self.parameters, OrderedDict):
            keys = sorted(self.parameters.keys()) # to ensure the order is always the same
        else:
            keys = self.parameters.keys()

        for k in keys:
            v = self.parameters[k].value
            par_fmt = "{}"
            post = ""

            if hasattr(v,'unit'):
                post = " {}".format(v.unit)
                v = v.value

            if isinstance(v, float):
                if v == 0:
                    par_fmt = "{:.0f}"
                elif np.log10(v) < -2 or np.log10(v) > 5:
                    par_fmt = "{:.2e}"
                else:
                    par_fmt = "{:.2f}"

            elif isinstance(v, int) and np.log10(v) > 5:
                par_fmt = "{:.2e}"

            pars += ("{}=" + par_fmt + post).format(k,v) + ", "

        if isinstance(self.units, DimensionlessUnitSystem):
            return "<{}: {} (dimensionless)>".format(self.__class__.__name__, pars.rstrip(", "))
        else:
            return "<{}: {} ({})>".format(self.__class__.__name__, pars.rstrip(", "), ",".join(map(str, self.units._core_units)))

    def __str__(self):
        return self.__class__.__name__

    # ========================================================================
    # Convenience methods that do fancy things
    #
    def plot_contours(self, grid, ax=None, labels=None, subplots_kw=dict(), **kwargs):
        """
        Plot equipotentials contours. Computes the potential value on a grid
        (specified by the array `grid`).

        .. warning::

            Right now the grid input must be arrays and must already be in
            the unit system of the potential. Quantity support is coming...

        Parameters
        ----------
        grid : tuple
            Coordinate grids or slice value for each dimension. Should be a
            tuple of 1D arrays or numbers.
        ax : matplotlib.Axes (optional)
        labels : iterable (optional)
            List of axis labels.
        subplots_kw : dict
            kwargs passed to matplotlib's subplots() function if an axes object
            is not specified.
        kwargs : dict
            kwargs passed to either contourf() or plot().

        Returns
        -------
        fig : `~matplotlib.Figure`

        """

        import matplotlib.pyplot as plt
        from matplotlib import cm

        # figure out which elements are iterable, which are numeric
        _grids = []
        _slices = []
        for ii,g in enumerate(grid):
            if isiterable(g):
                _grids.append((ii,g))
            else:
                _slices.append((ii,g))

        # figure out the dimensionality
        ndim = len(_grids)

        # if ndim > 2, don't know how to handle this!
        if ndim > 2:
            raise ValueError("ndim > 2: you can only make contours on a 2D grid. For other "
                             "dimensions, you have to specify values to slice.")

        if ax is None:
            # default figsize
            fig, ax = plt.subplots(1, 1, **subplots_kw)
        else:
            fig = ax.figure

        if ndim == 1:
            # 1D curve
            x1 = _grids[0][1]
            r = np.zeros((len(_grids) + len(_slices), len(x1)))
            r[_grids[0][0]] = x1

            for ii,slc in _slices:
                r[ii] = slc

            Z = self.value(r*self.units['length']).value
            ax.plot(x1, Z, **kwargs)

            if labels is not None:
                ax.set_xlabel(labels[0])
                ax.set_ylabel("potential")
        else:
            # 2D contours
            x1,x2 = np.meshgrid(_grids[0][1], _grids[1][1])
            shp = x1.shape
            x1,x2 = x1.ravel(), x2.ravel()

            r = np.zeros((len(_grids) + len(_slices), len(x1)))
            r[_grids[0][0]] = x1
            r[_grids[1][0]] = x2

            for ii,slc in _slices:
                r[ii] = slc

            Z = self.value(r*self.units['length']).value

            # make default colormap not suck
            cmap = kwargs.pop('cmap', cm.Blues)
            cs = ax.contourf(x1.reshape(shp), x2.reshape(shp), Z.reshape(shp),
                             cmap=cmap, **kwargs)

            if labels is not None:
                ax.set_xlabel(labels[0])
                ax.set_ylabel(labels[1])

        return fig

    def plot_densty_contours(self, grid, ax=None, labels=None, subplots_kw=dict(), **kwargs):
        """
        Plot density contours. Computes the density on a grid
        (specified by the array `grid`).

        .. warning::

            Right now the grid input must be arrays and must already be in
            the unit system of the potential. Quantity support is coming...

        Parameters
        ----------
        grid : tuple
            Coordinate grids or slice value for each dimension. Should be a
            tuple of 1D arrays or numbers.
        ax : matplotlib.Axes (optional)
        labels : iterable (optional)
            List of axis labels.
        subplots_kw : dict
            kwargs passed to matplotlib's subplots() function if an axes object
            is not specified.
        kwargs : dict
            kwargs passed to either contourf() or plot().

        Returns
        -------
        fig : `~matplotlib.Figure`

        """

        import matplotlib.pyplot as plt
        from matplotlib import cm

        # figure out which elements are iterable, which are numeric
        _grids = []
        _slices = []
        for ii,g in enumerate(grid):
            if isiterable(g):
                _grids.append((ii,g))
            else:
                _slices.append((ii,g))

        # figure out the dimensionality
        ndim = len(_grids)

        # if ndim > 2, don't know how to handle this!
        if ndim > 2:
            raise ValueError("ndim > 2: you can only make contours on a 2D grid. For other "
                             "dimensions, you have to specify values to slice.")

        if ax is None:
            # default figsize
            fig, ax = plt.subplots(1, 1, **subplots_kw)
        else:
            fig = ax.figure

        if ndim == 1:
            # 1D curve
            x1 = _grids[0][1]
            r = np.zeros((len(x1), len(_grids) + len(_slices)))
            r[:,_grids[0][0]] = x1

            for ii,slc in _slices:
                r[:,ii] = slc

            Z = self.density(r*self.units['length']).value
            ax.plot(x1, Z, **kwargs)

            if labels is not None:
                ax.set_xlabel(labels[0])
                ax.set_ylabel("potential")
        else:
            # 2D contours
            x1,x2 = np.meshgrid(_grids[0][1], _grids[1][1])
            shp = x1.shape
            x1,x2 = x1.ravel(), x2.ravel()

            r = np.zeros((len(x1), len(_grids) + len(_slices)))
            r[:,_grids[0][0]] = x1
            r[:,_grids[1][0]] = x2

            for ii,slc in _slices:
                r[:,ii] = slc

            Z = self.density(r*self.units['length']).value

            # make default colormap not suck
            cmap = kwargs.pop('cmap', cm.Blues)
            cs = ax.contourf(x1.reshape(shp), x2.reshape(shp), Z.reshape(shp),
                             cmap=cmap, **kwargs)
            # cs.cmap.set_under('w')
            # cs.cmap.set_over('k')

            if labels is not None:
                ax.set_xlabel(labels[0])
                ax.set_ylabel(labels[1])

        return fig

    def integrate_orbit(self, w0, Integrator=LeapfrogIntegrator,
                        Integrator_kwargs=dict(), cython_if_possible=True,
                        **time_spec):
        """
        Integrate an orbit in the current potential using the integrator class
        provided. Uses same time specification as `Integrator.run()` -- see
        the documentation for `gala.integrate` for more information.

        Parameters
        ----------
        w0 : `~gala.dynamics.PhaseSpacePosition`, array_like
            Initial conditions.
        Integrator : `~gala.integrate.Integrator` (optional)
            Integrator class to use.
        Integrator_kwargs : dict (optional)
            Any extra keyword argumets to pass to the integrator class
            when initializing. Only works in non-Cython mode.
        cython_if_possible : bool (optional)
            If there is a Cython version of the integrator implemented,
            and the potential object has a C instance, using Cython
            will be *much* faster.
        **time_spec
            Specification of how long to integrate. See documentation
            for `~gala.integrate.parse_time_specification`.

        Returns
        -------
        orbit : `~gala.dynamics.CartesianOrbit`

        """

        if not isinstance(w0, CartesianPhaseSpacePosition):
            w0 = np.asarray(w0)
            ndim = w0.shape[0]//2
            w0 = CartesianPhaseSpacePosition(pos=w0[:ndim],
                                             vel=w0[ndim:])

        ndim = w0.ndim
        arr_w0 = w0.w(self.units)
        if hasattr(self, 'c_instance') and cython_if_possible:
            # WARNING TO SELF: this transpose is there because the Cython
            #   functions expect a shape: (norbits, ndim)
            arr_w0 = np.ascontiguousarray(arr_w0.T)

            # array of times
            from ..integrate.timespec import parse_time_specification
            t = np.ascontiguousarray(parse_time_specification(self.units, **time_spec))

            if Integrator == LeapfrogIntegrator:
                from ..integrate.cyintegrators import leapfrog_integrate_potential
                t,w = leapfrog_integrate_potential(self.c_instance, arr_w0, t)

            elif Integrator == DOPRI853Integrator:
                from ..integrate.cyintegrators import dop853_integrate_potential
                t,w = dop853_integrate_potential(self.c_instance, arr_w0, t,
                                                 Integrator_kwargs.get('atol', 1E-10),
                                                 Integrator_kwargs.get('rtol', 1E-10),
                                                 Integrator_kwargs.get('nmax', 0))
            else:
                raise ValueError("Cython integration not supported for '{}'".format(Integrator))

            # because shape is different from normal integrator return
            w = np.rollaxis(w, -1)
            if w.shape[-1] == 1:
                w = w[...,0]

        else:
            def acc(t, w):
                return np.vstack((w[ndim:], -self._gradient(w[:ndim], t=t)))
            integrator = Integrator(acc, func_units=self.units, **Integrator_kwargs)
            orbit = integrator.run(w0, **time_spec)
            orbit.potential = self
            return orbit

        try:
            tunit = self.units['time']
        except (TypeError, AttributeError):
            tunit = u.dimensionless_unscaled
        return CartesianOrbit.from_w(w=w, units=self.units, t=t*tunit, potential=self)

    def total_energy(self, x, v):
        """
        Compute the total energy (per unit mass) of a point in phase-space
        in this potential. Assumes the last axis of the input position /
        velocity is the dimension axis, e.g., for 100 points in 3-space,
        the arrays should have shape (100,3).

        Parameters
        ----------
        x : array_like, numeric
            Position.
        v : array_like, numeric
            Velocity.
        """
        warnings.warn("Use the energy methods on Orbit objects instead. In a future "
                      "release this will be removed.", DeprecationWarning)

        v = atleast_2d(v, insert_axis=1)
        return self.value(x) + 0.5*np.sum(v**2, axis=0)

    def save(self, f):
        """
        Save the potential to a text file. See :func:`~gala.potential.save`
        for more information.

        Parameters
        ----------
        f : str, file_like
            A filename or file-like object to write the input potential object to.

        """
        from .io import save
        save(self, f)

class CompositePotential(PotentialBase, OrderedDict):
    """
    A potential composed of several distinct components. For example,
    two point masses or a galactic disk and halo, each with their own
    potential model.

    A `CompositePotential` is created like a Python dictionary, e.g.::

        >>> p1 = SomePotential(func1) # doctest: +SKIP
        >>> p2 = SomePotential(func2) # doctest: +SKIP
        >>> cp = CompositePotential(component1=p1, component2=p2) # doctest: +SKIP

    This object actually acts like an `OrderedDict`, so if you want to
    preserve the order of the potential components, use::

        >>> cp = CompositePotential() # doctest: +SKIP
        >>> cp['component1'] = p1 # doctest: +SKIP
        >>> cp['component2'] = p2 # doctest: +SKIP

    You can also use any of the built-in `Potential` classes as
    components::

        >>> from gala.potential import HernquistPotential
        >>> cp = CompositePotential()
        >>> cp['spheroid'] = HernquistPotential(m=1E11, c=10., units=(u.kpc,u.Myr,u.Msun,u.radian))

    """
    def __init__(self, *args, **kwargs):
        self._units = None

        if len(args) > 0 and isinstance(args[0], list):
            for k,v in args[0]:
                kwargs[k] = v
        else:
            for i,v in args:
                kwargs[str(i)] = v

        self.lock = False
        for v in kwargs.values():
            self._check_component(v)

        OrderedDict.__init__(self, **kwargs)

    def __setitem__(self, key, value):
        self._check_component(value)
        super(CompositePotential, self).__setitem__(key, value)

    def _check_component(self, p):
        if not isinstance(p, PotentialBase):
            raise TypeError("Potential components may only be Potential "
                            "objects, not {0}.".format(type(p)))

        if self.units is None:
            self._units = p.units

        else:
            if sorted([str(x) for x in self.units]) != sorted([str(x) for x in p.units]):
                raise ValueError("Unit system of new potential component must match "
                                 "unit systems of other potential components.")

        if self.lock:
            raise ValueError("Potential object is locked - new components can only be"
                             " added to unlocked potentials.")

    @property
    def units(self):  # read-only
        return self._units

    @property
    def parameters(self):
        params = dict()
        for k,v in self.items():
            params[k] = v.parameters
        return ImmutableDict(**params)

    def _value(self, q, t=0.):
        return sum([p._value(q, t) for p in self.values()])

    def _gradient(self, q, t=0.):
        return sum([p._gradient(q, t) for p in self.values()])

    def _hessian(self, w, t=0.):
        return sum([p._hessian(w, t) for p in self.values()])

    def _density(self, q, t=0.):
        return sum([p._density(q, t) for p in self.values()])

    def __repr__(self):
        return "<CompositePotential {}>".format(",".join(self.keys()))



# coding: utf-8
# cython: boundscheck=False
# cython: nonecheck=False
# cython: cdivision=True
# cython: wraparound=False
# cython: profile=False

""" DOP853 integration in Cython. """

from __future__ import division, print_function

__author__ = "adrn <adrn@astro.columbia.edu>"

# Third-party
import numpy as np
cimport numpy as np
np.import_array()

from libc.stdio cimport printf
from cpython.exc cimport PyErr_CheckSignals

from ...potential.cpotential cimport CPotentialWrapper

cdef extern from "src/cpotential.h":
    ctypedef struct CPotential:
        pass
    void c_gradient(CPotential *p, double t, double *q, double *grad) nogil

cdef extern from "dopri/dop853.h":
    ctypedef void (*FcnEqDiff)(unsigned n, double x, double *y, double *f,
                              CPotential *p, unsigned norbits) nogil
    ctypedef void (*SolTrait)(long nr, double xold, double x, double* y, unsigned n, int* irtrn)

    # See dop853.h for full description of all input parameters
    int dop853 (unsigned n, FcnEqDiff fn,
                CPotential *p, unsigned n_orbits,
                double x, double* y, double xend,
                double* rtoler, double* atoler, int itoler, SolTrait solout,
                int iout, FILE* fileout, double uround, double safe, double fac1,
                double fac2, double beta, double hmax, double h, long nmax, int meth,
                long nstiff, unsigned nrdens, unsigned* icont, unsigned licont)

    void Fwrapper (unsigned ndim, double t, double *w, double *f,
                   CPotential *p, unsigned norbits)
    double six_norm (double *x)

cdef extern from "stdio.h":
    ctypedef struct FILE
    FILE *stdout

cdef void solout(long nr, double xold, double x, double* y, unsigned n, int* irtrn):
    # TODO: see here for example in FORTRAN:
    #   http://www.unige.ch/~hairer/prog/nonstiff/dr_dop853.f
    pass

cpdef dop853_integrate_potential(CPotentialWrapper cp, double[:,::1] w0,
                                 double[::1] t,
                                 double atol=1E-10, double rtol=1E-10, int nmax=0):
    """
    CAUTION: Interpretation of axes is different here! We need the
    arrays to be C ordered and easy to iterate over, so here the
    axes are (norbits, ndim).

    TODO: add option for a callback function to be called at each step
    """
    cdef:
        int i, j, k
        int res, iout
        unsigned norbits = w0.shape[0]
        unsigned ndim = w0.shape[1]

        # define full array of times
        int ntimes = len(t)
        double dt0 = t[1]-t[0]
        double[::1] w = np.empty(ndim*norbits)

        # Note: icont not needed because nrdens == ndim
        double[:,:,::1] all_w = np.empty((ntimes,norbits,ndim))

    # store initial conditions
    for i in range(norbits):
        for k in range(ndim):
            w[i*ndim + k] = w0[i,k]
            all_w[0,i,k] = w0[i,k]

    # TODO: any way to support dense output?
    iout = 0  # no solout calls

    for j in range(1,ntimes,1):
        res = dop853(ndim*norbits, <FcnEqDiff> Fwrapper,
                     &(cp.cpotential), norbits,
                     t[j-1], &w[0], t[j], &rtol, &atol, 0, solout, iout,
                     NULL, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, dt0, nmax, 0, 1, 0, NULL, 0);

        if res == -1:
            raise RuntimeError("Input is not consistent.")
        elif res == -2:
            raise RuntimeError("Larger nmax is needed.")
        elif res == -3:
            raise RuntimeError("Step size becomes too small.")
        elif res == -4:
            raise RuntimeError("The problem is probably stiff (interrupted).")

        for k in range(ndim):
            for i in range(norbits):
                all_w[j,i,k] = w[i*ndim + k]

        PyErr_CheckSignals()

    return np.asarray(t), np.asarray(all_w)



# coding: utf-8
# cython: boundscheck=False
# cython: nonecheck=False
# cython: cdivision=True
# cython: wraparound=False
# cython: profile=False

""" DOP853 integration in Cython. """

from __future__ import division, print_function

__author__ = "adrn <adrn@astro.columbia.edu>"

# Third-party
import numpy as np
cimport numpy as np
np.import_array()

from libc.stdio cimport printf
from libc.math cimport log

from ...potential.cpotential cimport CPotentialWrapper

cdef extern from "src/cpotential.h":
    ctypedef struct CPotential:
        pass
    void c_gradient(CPotential *p, double t, double *q, double *grad) nogil

cdef extern from "dopri/dop853.h":
    ctypedef void (*FcnEqDiff)(unsigned n, double x, double *y, double *f,
                              CPotential *p, unsigned norbits) nogil
    ctypedef void (*SolTrait)(long nr, double xold, double x, double* y, unsigned n, int* irtrn)

    # See dop853.h for full description of all input parameters
    int dop853 (unsigned n, FcnEqDiff fn,
                CPotential *p, unsigned n_orbits,
                double x, double* y, double xend,
                double* rtoler, double* atoler, int itoler, SolTrait solout,
                int iout, FILE* fileout, double uround, double safe, double fac1,
                double fac2, double beta, double hmax, double h, long nmax, int meth,
                long nstiff, unsigned nrdens, unsigned* icont, unsigned licont)

    void Fwrapper (unsigned ndim, double t, double *w, double *f,
                   CPotential *p, unsigned norbits)
    double six_norm (double *x)

cdef extern from "stdio.h":
    ctypedef struct FILE
    FILE *stdout

cdef void solout(long nr, double xold, double x, double* y, unsigned n, int* irtrn):
    # TODO: see here for example in FORTRAN:
    #   http://www.unige.ch/~hairer/prog/nonstiff/dr_dop853.f
    pass

cpdef dop853_lyapunov_max(CPotentialWrapper cp, double[::1] w0,
                          double dt, int n_steps, double t0,
                          double d0, int n_steps_per_pullback, int noffset_orbits,
                          double atol=1E-10, double rtol=1E-10, int nmax=0):
    cdef:
        int i, j, k, jiter
        int res
        unsigned ndim = w0.size
        unsigned norbits = noffset_orbits + 1
        unsigned niter = n_steps // n_steps_per_pullback
        double[::1] w = np.empty(norbits*ndim)

        # define full array of times
        double t_end = (<double>n_steps) * dt
        double[::1] t = np.linspace(t0, t_end, n_steps) # TODO: should be n_steps+1

        double d1_mag, norm
        double[:,::1] d1 = np.empty((norbits,ndim))
        double[:,::1] LEs = np.zeros((niter,noffset_orbits))
        double[:,:,::1] all_w = np.zeros((n_steps,norbits,ndim))

        # temp stuff
        double[:,::1] d0_vec = np.random.uniform(size=(noffset_orbits,ndim))

    # store initial conditions
    for i in range(norbits):
        if i == 0:  # store initial conditions for parent orbit
            for k in range(ndim):
                all_w[0,i,k] = w0[k]
                w[i*ndim + k] = all_w[0,i,k]

        else:  # offset orbits
            norm = np.linalg.norm(d0_vec[i-1])
            for k in range(ndim):
                d0_vec[i-1,k] *= d0/norm  # rescale offset vector

                all_w[0,i,k] = w0[k] + d0_vec[i-1,k]
                w[i*ndim + k] = all_w[0,i,k]

    # dummy counter for storing Lyapunov stuff, which only happens every few steps
    jiter = 0
    for j in range(1,n_steps,1):
        res = dop853(ndim*norbits, <FcnEqDiff> Fwrapper,
                     &(cp.cpotential), norbits,
                     t[j-1], &w[0], t[j], &rtol, &atol, 0, solout, 0,
                     NULL, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, dt, nmax, 0, 1, 0, NULL, 0);

        if res == -1:
            raise RuntimeError("Input is not consistent.")
        elif res == -2:
            raise RuntimeError("Larger nmax is needed.")
        elif res == -3:
            raise RuntimeError("Step size becomes too small.")
        elif res == -4:
            raise RuntimeError("The problem is probably stff (interrupted).")

        # store position of main orbit
        for i in range(norbits):
            for k in range(ndim):
                all_w[j,i,k] = w[i*ndim + k]

        if (j % n_steps_per_pullback) == 0:
            # get magnitude of deviation vector
            for i in range(1,norbits):
                for k in range(ndim):
                    d1[i,k] = w[i*ndim + k] - w[k]

                d1_mag = six_norm(&d1[i,0])
                LEs[jiter,i-1] = log(d1_mag / d0)

                # renormalize offset orbits
                for k in range(ndim):
                    w[i*ndim + k] = w[k] + d0 * d1[i,k] / d1_mag

            jiter += 1

    LEs = np.array([np.sum(LEs[:j],axis=0)/t[j*n_steps_per_pullback] for j in range(1,niter)])
    return np.asarray(t), np.asarray(all_w), np.asarray(LEs)

cpdef dop853_lyapunov_max_dont_save(CPotentialWrapper cp, double[::1] w0,
                                    double dt, int n_steps, double t0,
                                    double d0, int n_steps_per_pullback, int noffset_orbits,
                                    double atol=1E-10, double rtol=1E-10, int nmax=0):
    cdef:
        int i, j, k, jiter
        int res
        unsigned ndim = w0.size
        unsigned norbits = noffset_orbits + 1
        unsigned niter = n_steps // n_steps_per_pullback
        double[::1] w = np.empty(norbits*ndim)

        # define full array of times
        double t_end = (<double>n_steps) * dt
        double[::1] t = np.linspace(t0, t_end, n_steps) # TODO: should be n_steps+1

        double d1_mag, norm
        double[:,::1] d1 = np.empty((norbits,ndim))
        double[:,::1] LEs = np.zeros((niter,noffset_orbits))

        # temp stuff
        double[:,::1] d0_vec = np.random.uniform(size=(noffset_orbits,ndim))

    # store initial conditions
    for i in range(norbits):
        if i == 0:  # store initial conditions for parent orbit
            for k in range(ndim):
                w[i*ndim + k] = w0[k]

        else:  # offset orbits
            norm = np.linalg.norm(d0_vec[i-1])
            for k in range(ndim):
                d0_vec[i-1,k] *= d0/norm  # rescale offset vector
                w[i*ndim + k] = w0[k] + d0_vec[i-1,k]

    # dummy counter for storing Lyapunov stuff, which only happens every few steps
    jiter = 0
    for j in range(1,n_steps,1):
        res = dop853(ndim*norbits, <FcnEqDiff> Fwrapper,
                     &(cp.cpotential), norbits,
                     t[j-1], &w[0], t[j], &rtol, &atol, 0, solout, 0,
                     NULL, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, dt, nmax, 0, 1, 0, NULL, 0);

        if res == -1:
            raise RuntimeError("Input is not consistent.")
        elif res == -2:
            raise RuntimeError("Larger nmax is needed.")
        elif res == -3:
            raise RuntimeError("Step size becomes too small.")
        elif res == -4:
            raise RuntimeError("The problem is probably stff (interrupted).")

        if (j % n_steps_per_pullback) == 0:
            # get magnitude of deviation vector
            for i in range(1,norbits):
                for k in range(ndim):
                    d1[i,k] = w[i*ndim + k] - w[k]

                d1_mag = six_norm(&d1[i,0])
                LEs[jiter,i-1] = log(d1_mag / d0)

                # renormalize offset orbits
                for k in range(ndim):
                    w[i*ndim + k] = w[k] + d0 * d1[i,k] / d1_mag

            jiter += 1

    LEs = np.array([np.sum(LEs[:j],axis=0)/t[j*n_steps_per_pullback] for j in range(1,niter)])
    return np.asarray(LEs)



import glob
import imp
import os
import pkgutil
import re
import sys
import tarfile

from . import *


PY3 = sys.version_info[0] == 3

if PY3:
    _text_type = str
else:
    _text_type = unicode


_DEV_VERSION_RE = re.compile(r'\d+\.\d+(?:\.\d+)?\.dev(\d+)')


TEST_VERSION_SETUP_PY = """\
#!/usr/bin/env python

from setuptools import setup

NAME = 'apyhtest_eva'
VERSION = {version!r}
RELEASE = 'dev' not in VERSION

from astropy_helpers.git_helpers import get_git_devstr
from astropy_helpers.version_helpers import generate_version_py

if not RELEASE:
    VERSION += get_git_devstr(False)

generate_version_py(NAME, VERSION, RELEASE, False, uses_git=not RELEASE)

setup(name=NAME, version=VERSION, packages=['apyhtest_eva'])
"""


TEST_VERSION_INIT = """\
try:
    from .version import version as __version__
    from .version import githash as __githash__
except ImportError:
    __version__ = __githash__ = ''
"""


@pytest.fixture
def version_test_package(tmpdir, request):
    def make_test_package(version='42.42.dev'):
        test_package = tmpdir.mkdir('test_package')
        test_package.join('setup.py').write(
            TEST_VERSION_SETUP_PY.format(version=version))
        test_package.mkdir('apyhtest_eva').join('__init__.py').write(TEST_VERSION_INIT)
        with test_package.as_cwd():
            run_cmd('git', ['init'])
            run_cmd('git', ['add', '--all'])
            run_cmd('git', ['commit', '-m', 'test package'])

        if '' in sys.path:
            sys.path.remove('')

        sys.path.insert(0, '')

        def finalize():
            cleanup_import('apyhtest_eva')

        request.addfinalizer(finalize)

        return test_package

    return make_test_package


def test_update_git_devstr(version_test_package, capsys):
    """Tests that the commit number in the package's version string updates
    after git commits even without re-running setup.py.
    """

    # We have to call version_test_package to actually create the package
    test_pkg = version_test_package()

    with test_pkg.as_cwd():
        run_setup('setup.py', ['--version'])

        stdout, stderr = capsys.readouterr()
        version = stdout.strip()

        m = _DEV_VERSION_RE.match(version)
        assert m, (
            "Stdout did not match the version string pattern:"
            "\n\n{0}\n\nStderr:\n\n{1}".format(stdout, stderr))
        revcount = int(m.group(1))

        import apyhtest_eva
        assert apyhtest_eva.__version__ == version

        # Make a silly git commit
        with open('.test', 'w'):
            pass

        run_cmd('git', ['add', '.test'])
        run_cmd('git', ['commit', '-m', 'test'])

        import apyhtest_eva.version
        imp.reload(apyhtest_eva.version)

    # Previously this checked packagename.__version__, but in order for that to
    # be updated we also have to re-import _astropy_init which could be tricky.
    # Checking directly that the packagename.version module was updated is
    # sufficient:
    m = _DEV_VERSION_RE.match(apyhtest_eva.version.version)
    assert m
    assert int(m.group(1)) == revcount + 1

    # This doesn't test astropy_helpers.get_helpers.update_git_devstr directly
    # since a copy of that function is made in packagename.version (so that it
    # can work without astropy_helpers installed).  In order to get test
    # coverage on the actual astropy_helpers copy of that function just call it
    # directly and compare to the value in packagename
    from astropy_helpers.git_helpers import update_git_devstr

    newversion = update_git_devstr(version, path=str(test_pkg))
    assert newversion == apyhtest_eva.version.version


def test_version_update_in_other_repos(version_test_package, tmpdir):
    """
    Regression test for https://github.com/astropy/astropy-helpers/issues/114
    and for https://github.com/astropy/astropy-helpers/issues/107
    """

    test_pkg = version_test_package()

    with test_pkg.as_cwd():
        run_setup('setup.py', ['build'])

    # Add the path to the test package to sys.path for now
    sys.path.insert(0, str(test_pkg))
    try:
        import apyhtest_eva
        m = _DEV_VERSION_RE.match(apyhtest_eva.__version__)
        assert m
        correct_revcount = int(m.group(1))

        with tmpdir.as_cwd():
            testrepo = tmpdir.mkdir('testrepo')
            testrepo.chdir()
            # Create an empty git repo
            run_cmd('git', ['init'])

            import apyhtest_eva.version
            imp.reload(apyhtest_eva.version)
            m = _DEV_VERSION_RE.match(apyhtest_eva.version.version)
            assert m
            assert int(m.group(1)) == correct_revcount
            correct_revcount = int(m.group(1))

            # Add several commits--more than the revcount for the apyhtest_eva package
            for idx in range(correct_revcount + 5):
                test_filename = '.test' + str(idx)
                testrepo.ensure(test_filename)
                run_cmd('git', ['add', test_filename])
                run_cmd('git', ['commit', '-m', 'A message'])

            import apyhtest_eva.version
            imp.reload(apyhtest_eva.version)
            m = _DEV_VERSION_RE.match(apyhtest_eva.version.version)
            assert m
            assert int(m.group(1)) == correct_revcount
            correct_revcount = int(m.group(1))
    finally:
        sys.path.remove(str(test_pkg))


@pytest.mark.parametrize('version', ['1.0.dev', '1.0'])
def test_installed_git_version(version_test_package, version, tmpdir, capsys):
    """
    Test for https://github.com/astropy/astropy-helpers/issues/87

    Ensures that packages installed with astropy_helpers have a correct copy
    of the git hash of the installed commit.
    """

    # To test this, it should suffice to build a source dist, unpack it
    # somewhere outside the git repository, and then do a build and import
    # from the build directory--no need to "install" as such

    test_pkg = version_test_package(version)

    with test_pkg.as_cwd():
        run_setup('setup.py', ['build'])

        try:
            import apyhtest_eva
            githash = apyhtest_eva.__githash__
            assert githash and isinstance(githash, _text_type)
            # Ensure that it does in fact look like a git hash and not some
            # other arbitrary string
            assert re.match(r'[0-9a-f]{40}', githash)
        finally:
            cleanup_import('apyhtest_eva')

        run_setup('setup.py', ['sdist', '--dist-dir=dist', '--formats=gztar'])

        tgzs = glob.glob(os.path.join('dist', '*.tar.gz'))
        assert len(tgzs) == 1

        tgz = test_pkg.join(tgzs[0])

    build_dir = tmpdir.mkdir('build_dir')
    tf = tarfile.open(str(tgz), mode='r:gz')
    tf.extractall(str(build_dir))

    with build_dir.as_cwd():
        pkg_dir = glob.glob('apyhtest_eva-*')[0]
        os.chdir(pkg_dir)
        run_setup('setup.py', ['build'])

        try:
            import apyhtest_eva
            loader = pkgutil.get_loader('apyhtest_eva')
            # Ensure we are importing the 'packagename' that was just unpacked
            # into the build_dir
            if sys.version_info[:2] != (3, 3):
                # Skip this test on Python 3.3 wherein the SourceFileLoader
                # has a bug where get_filename() does not return an absolute
                # path
                assert loader.get_filename().startswith(str(build_dir))
            assert apyhtest_eva.__githash__ == githash
        finally:
            cleanup_import('apyhtest_eva')



# coding: utf-8
# cython: boundscheck=False
# cython: debug=False
# cython: nonecheck=False
# cython: cdivision=True
# cython: wraparound=False
# cython: profile=False

""" Generate mock streams. """

from __future__ import division, print_function

__author__ = "adrn <adrn@astro.columbia.edu>"

# Third-party
import numpy as np
cimport numpy as np
np.import_array()

from libc.math cimport M_PI, sqrt
from cpython.exc cimport PyErr_CheckSignals

from ...potential.cpotential cimport CPotentialWrapper
from ._coord cimport (sat_rotation_matrix, to_sat_coords, from_sat_coords,
                      cyl_to_car, car_to_cyl)

__all__ = ['_mock_stream_dop853']

cdef extern from "src/cpotential.h":
    ctypedef struct CPotential:
        pass
    void c_gradient(CPotential *p, double t, double *q, double *grad) nogil
    double c_d2_dr2(CPotential *p, double t, double *q, double *epsilon) nogil

cdef extern from "dopri/dop853.h":
    ctypedef void (*FcnEqDiff)(unsigned n, double x, double *y, double *f,
                              CPotential *p, unsigned norbits) nogil
    ctypedef void (*SolTrait)(long nr, double xold, double x, double* y, unsigned n, int* irtrn)

    # See dop853.h for full description of all input parameters
    int dop853 (unsigned n, FcnEqDiff fn,
                CPotential *p, unsigned n_orbits,
                double x, double* y, double xend,
                double* rtoler, double* atoler, int itoler, SolTrait solout,
                int iout, FILE* fileout, double uround, double safe, double fac1,
                double fac2, double beta, double hmax, double h, long nmax, int meth,
                long nstiff, unsigned nrdens, unsigned* icont, unsigned licont)

    void Fwrapper (unsigned ndim, double t, double *w, double *f,
                   CPotential *p, unsigned norbits)

cdef extern from "stdio.h":
    ctypedef struct FILE
    FILE *stdout

cpdef _mock_stream_dop853(CPotentialWrapper cp, double[::1] t, double[:,::1] prog_w,
                          int release_every,
                          _k_mean, _k_disp,
                          double G, _prog_mass,
                          double atol=1E-10, double rtol=1E-10, int nmax=0):
    """
    _mock_stream_dop853(cpotential, t, prog_w, release_every, k_mean, k_disp, G, prog_mass, atol, rtol, nmax)

    Generate a mock stellar stream using the Streakline method.

    Parameters
    ----------
    cpotential : `gala.potential._CPotential`
        An instance of a ``_CPotential`` representing the gravitational potential.
    t : `numpy.ndarray`
        An array of times. Should have shape ``(ntimesteps,)``.
    prog_w : `numpy.ndarray`
        The 6D coordinates for the orbit of the progenitor system at all times.
        Should have shape ``(ntimesteps,6)``.
    release_every : int
        Release particles at the Lagrange points every X timesteps.
    k_mean : `numpy.ndarray`
        Array of mean ``k`` values (see Fardal et al. 2015). These are used to determine
        the exact prescription for generating the mock stream. The components are for:
        ``(R,phi,z,vR,vphi,vz)``. If 1D, assumed constant in time. If 2D, time axis is axis 0.
    k_disp : `numpy.ndarray`
        Array of ``k`` value dispersions (see Fardal et al. 2015). These are used to determine
        the exact prescription for generating the mock stream. The components are for:
        ``(R,phi,z,vR,vphi,vz)``. If 1D, assumed constant in time. If 2D, time axis is axis 0.
    G : numeric
        The value of the gravitational constant, G, in the unit system used.
    prog_mass : float or `numpy.ndarray`
        The mass of the progenitor or the mass at each time. Should be a scalar or have
        shape ``(ntimesteps,)``.
    atol : numeric (optional)
        Passed to the integrator. Absolute tolerance parameter. Default is 1E-10.
    rtol : numeric (optional)
        Passed to the integrator. Relative tolerance parameter. Default is 1E-10.
    nmax : int (optional)
        Passed to the integrator.
    """
    cdef:
        int i, j, k # indexing
        int res # result from calling dop853
        int ntimes = t.shape[0] # number of times
        int nparticles # total number of test particles released

        unsigned ndim = prog_w.shape[1] # phase-space dimensionality
        unsigned ndim_2 = ndim / 2 # configuration-space dimensionality

        double dt0 = t[1] - t[0] # initial timestep
        double[::1] tmp = np.zeros(3) # temporary array

        double[::1] w_prime = np.zeros(6) # 6-position of stripped star
        double[::1] cyl = np.zeros(6) # 6-position in cylindrical coords
        double[::1] prog_w_prime = np.zeros(6) # 6-position of progenitor rotated
        double[::1] prog_cyl = np.zeros(6) # 6-position of progenitor in cylindrical coords

        # k-factors for parametrized model of Fardal et al. (2015)
        double[::1] ks = np.zeros(6)

        # used for figuring out how many orbits to integrate at any given release time
        unsigned this_ndim, this_norbits

        double Om # angular velocity squared
        double d, sigma_r # distance, dispersion in release positions
        double r_tide, menc, f # tidal radius, mass enclosed, f factor

        double[::1] eps = np.zeros(3) # used for 2nd derivative estimation
        double[:,::1] R = np.zeros((3,3)) # rotation matrix

        double[::1] prog_mass = np.ascontiguousarray(np.atleast_1d(_prog_mass))
        double[:,::1] k_mean = np.ascontiguousarray(np.atleast_2d(_k_mean))
        double[:,::1] k_disp = np.ascontiguousarray(np.atleast_2d(_k_disp))
        double[::1] mu_k
        double[::1] sigma_k

    # figure out how many particles are going to be released into the "stream"
    if ntimes % release_every == 0:
        nparticles = 2 * (ntimes // release_every)
    else:
        nparticles = 2 * (ntimes // release_every + 1)

    # container for only current positions of all particles
    cdef double[::1] w = np.empty(nparticles*ndim)

    # beginning times for each particle
    cdef double[::1] t1 = np.empty(nparticles)
    cdef double t_end = t[ntimes-1]

    # -------

    # copy over initial conditions from progenitor orbit to each streakline star
    i = 0
    for j in range(ntimes):
        if (j % release_every) != 0:
            continue

        for k in range(ndim):
            w[2*i*ndim + k] = prog_w[j,k]
            w[2*i*ndim + k + ndim] = prog_w[j,k]

        i += 1

    # now go back to each set of initial conditions and modify initial condition
    #   based on mock prescription
    i = 0
    for j in range(ntimes):
        if (j % release_every) != 0:
            continue

        t1[2*i] = t[j]
        t1[2*i+1] = t[j]

        if prog_mass.shape[0] == 1:
            M = prog_mass[0]
        else:
            M = prog_mass[j]

        if k_mean.shape[0] == 1:
            mu_k = k_mean[0]
            sigma_k = k_disp[0]
        else:
            mu_k = k_mean[j]
            sigma_k = k_disp[j]

        # angular velocity
        d = sqrt(prog_w[j,0]*prog_w[j,0] +
                 prog_w[j,1]*prog_w[j,1] +
                 prog_w[j,2]*prog_w[j,2])
        Om = np.linalg.norm(np.cross(prog_w[j,:3], prog_w[j,3:]) / d**2)

        # gradient of potential in radial direction
        f = Om*Om - c_d2_dr2(&(cp.cpotential), t[j], &prog_w[j,0], &eps[0])
        r_tide = (G*M / f)**(1/3.)

        # the rotation matrix to transform from satellite coords to normal
        sat_rotation_matrix(&prog_w[j,0], &R[0,0])
        to_sat_coords(&prog_w[j,0], &R[0,0], &prog_w_prime[0])
        car_to_cyl(&prog_w_prime[0], &prog_cyl[0])

        for k in range(6):
            if sigma_k[k] > 0:
                ks[k] = np.random.normal(mu_k[k], sigma_k[k])
            else:
                ks[k] = mu_k[k]

        # eject stars at tidal radius with same angular velocity as progenitor
        cyl[0] = prog_cyl[0] + ks[0]*r_tide
        cyl[1] = prog_cyl[1] + ks[1]*r_tide/prog_cyl[0]
        cyl[2] = ks[2]*r_tide/prog_cyl[0]
        cyl[3] = prog_cyl[3] + ks[3]*prog_cyl[3]
        cyl[4] = prog_cyl[4] + ks[0]*ks[4]*Om*r_tide
        cyl[5] = ks[5]*Om*r_tide
        cyl_to_car(&cyl[0], &w_prime[0])
        from_sat_coords(&w_prime[0], &R[0,0], &w[2*i*ndim])

        for k in range(6):
            if sigma_k[k] > 0:
                ks[k] = np.random.normal(mu_k[k], sigma_k[k])
            else:
                ks[k] = mu_k[k]

        cyl[0] = prog_cyl[0] - ks[0]*r_tide
        cyl[1] = prog_cyl[1] - ks[1]*r_tide/prog_cyl[0]
        cyl[2] = ks[2]*r_tide/prog_cyl[0]
        cyl[3] = prog_cyl[3] + ks[3]*prog_cyl[3]
        cyl[4] = prog_cyl[4] - ks[0]*ks[4]*Om*r_tide
        cyl[5] = ks[5]*Om*r_tide
        cyl_to_car(&cyl[0], &w_prime[0])
        from_sat_coords(&w_prime[0], &R[0,0], &w[2*i*ndim + ndim])

        i += 1

    for i in range(nparticles):
        res = dop853(ndim, <FcnEqDiff> Fwrapper,
                     &(cp.cpotential), 1,
                     t1[i], &w[i*ndim], t_end, &rtol, &atol, 0, NULL, 0,
                     NULL, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, dt0, nmax, 0, 1, 0, NULL, 0);

        if res == -1:
            raise RuntimeError("Input is not consistent.")
        elif res == -2:
            raise RuntimeError("Larger nmax is needed.")
        elif res == -3:
            raise RuntimeError("Step size becomes too small.")
        elif res == -4:
            raise RuntimeError("The problem is probably stff (interrupted).")

        PyErr_CheckSignals()

    return np.asarray(w).reshape(nparticles, ndim)



# coding: utf-8
"""
    Test the unit system.
"""

from __future__ import absolute_import, unicode_literals, division, print_function

__author__ = "adrn <adrn@astro.columbia.edu>"

# Third party
import astropy.units as u
from astropy.constants import G,c
import numpy as np
import pytest

# This package
from ..units import UnitSystem, DimensionlessUnitSystem

def test_create():
    # dumb
    usys = UnitSystem(u.kpc, u.Myr, u.radian, u.Msun)

    with pytest.raises(ValueError):
        UnitSystem(u.kpc, u.Myr, u.radian) # no mass

    with pytest.raises(ValueError):
        UnitSystem(u.kpc, u.Myr, u.Msun)

    with pytest.raises(ValueError):
        UnitSystem(u.kpc, u.radian, u.Msun)

    with pytest.raises(ValueError):
        UnitSystem(u.Myr, u.radian, u.Msun)

    usys = UnitSystem((u.kpc, u.Myr, u.radian, u.Msun))
    usys = UnitSystem(usys)

def test_constants():
    usys = UnitSystem(u.kpc, u.Myr, u.radian, u.Msun)
    assert np.allclose(usys.get_constant('G'), G.decompose([u.kpc, u.Myr, u.radian, u.Msun]).value)
    assert np.allclose(usys.get_constant('c'), c.decompose([u.kpc, u.Myr, u.radian, u.Msun]).value)

def test_decompose():
    usys = UnitSystem(u.kpc, u.Myr, u.radian, u.Msun, u.km/u.s)
    q = 15.*u.km/u.s
    assert q.decompose(usys).unit == u.kpc/u.Myr # uses the core units
    assert usys.decompose(q).unit == u.km/u.s

def test_dimensionless():
    usys = DimensionlessUnitSystem()
    assert usys['dimensionless'] == u.one
    assert usys['length'] == u.one

    with pytest.raises(ValueError):
        (15*u.kpc).decompose(usys)

    with pytest.raises(ValueError):
        usys.decompose(15*u.kpc)



# coding: utf-8

""" Transform a proper motion to/from Galactic to/from ICRS coordinates """

from __future__ import division, print_function

__author__ = "adrn <adrn@astro.columbia.edu>"

# Third-party
import numpy as np
import astropy.coordinates as coord

__all__ = ['pm_gal_to_icrs', 'pm_icrs_to_gal']

def pm_gal_to_icrs(coordinate, pm):
    r"""
    Convert proper motion in Galactic coordinates (l,b) to
    ICRS coordinates (RA, Dec).

    Parameters
    ----------
    coordinate : :class:`~astropy.coordinates.SkyCoord`, :class:`~astropy.coordinates.BaseCoordinateFrame`
        An instance of an Astropy coordinate object. Can be in any
        frame that is transformable to ICRS coordinates.
    pm : :class:`~astropy.units.Quantity`, iterable
        Full description of proper motion in Galactic longitude and
        latitude. Can either be a tuple of two
        :class:`~astropy.units.Quantity` objects or a single
        :class:`~astropy.units.Quantity` with shape (2,N).
        The proper motion in longitude is assumed to be multipled by
        cosine of Galactic latitude, :math:`\mu_l\cos b`.

    Returns
    -------
    pm : :class:`~astropy.units.Quantity`
        An astropy :class:`~astropy.units.Quantity` object specifying the
        proper motion vector array in ICRS coordinates. Will have shape
        (2,N).

    Examples
    --------

        >>> import astropy.units as u
        >>> import astropy.coordinates as coord
        >>> c = coord.SkyCoord(ra=196.5*u.degree, dec=-10.33*u.deg, distance=16.2*u.kpc)
        >>> pm = [-1.53, 3.5]*u.mas/u.yr
        >>> pm_gal_to_icrs(c, pm) # doctest: +FLOAT_CMP
        <Quantity [-1.84741767, 3.34334366] mas / yr>

    """

    g = coordinate.transform_to(coord.Galactic)
    i = coordinate.transform_to(coord.ICRS)

    mulcosb,mub = map(np.atleast_1d, pm)
    shape = mulcosb.shape

    # coordinates of NGP
    ag = coord.Galactic._ngp_J2000.ra
    dg = coord.Galactic._ngp_J2000.dec

    # used for the transformation matrix from Bovy (2011)
    cosphi = (np.sin(dg) - np.sin(i.dec)*np.sin(g.b)) / np.cos(g.b) / np.cos(i.dec)
    sinphi = np.sin(i.ra - ag) * np.cos(dg) / np.cos(g.b)

    R = np.zeros((2,2)+shape)
    R[0,0] = cosphi
    R[0,1] = -sinphi
    R[1,0] = sinphi
    R[1,1] = cosphi

    mu = np.vstack((mulcosb.value[None], mub.to(mulcosb.unit).value[None]))
    # new_mu = np.zeros_like(mu)
    # for i in range(n):
    #     new_mu[:,i] = R[...,i].dot(mu[:,i])
    new_mu = np.einsum('ijk...,jk...->ik...',R,mu)

    if coordinate.isscalar:
        return new_mu.reshape((2,))*mulcosb.unit
    else:
        return new_mu.reshape((2,) + mulcosb.shape)*mulcosb.unit

def pm_icrs_to_gal(coordinate, pm):
    r"""
    Convert proper motion in ICRS coordinates (RA, Dec) to
    Galactic coordinates (l,b).

    Parameters
    ----------
    coordinate : :class:`~astropy.coordinates.SkyCoord`, :class:`~astropy.coordinates.BaseCoordinateFrame`
        An instance of an Astropy coordinate object. Can be in any
        frame that is transformable to ICRS coordinates.
    pm : :class:`~astropy.units.Quantity`, iterable
        Full description of proper motion in Right ascension (RA) and
        declination (Dec). Can either be a tuple of two
        :class:`~astropy.units.Quantity` objects or a single
        :class:`~astropy.units.Quantity` with shape (2,N).
        The proper motion in RA is assumed to be multipled by
        cosine of declination, :math:`\mu_\alpha\cos\delta`.

    Returns
    -------
    pm : :class:`~astropy.units.Quantity`
        An astropy :class:`~astropy.units.Quantity` object specifying the
        proper motion vector array in Galactic coordinates. Will have shape
        (2,N).

    Examples
    --------

        >>> import astropy.units as u
        >>> import astropy.coordinates as coord
        >>> c = coord.SkyCoord(ra=196.5*u.degree, dec=-10.33*u.deg, distance=16.2*u.kpc)
        >>> pm = [-1.84741767, 3.34334366]*u.mas/u.yr
        >>> pm_icrs_to_gal(c, pm) # doctest: +FLOAT_CMP
        <Quantity [-1.52999988, 3.49999973] mas / yr>

    """

    g = coordinate.transform_to(coord.Galactic)
    i = coordinate.transform_to(coord.ICRS)

    muacosd,mud = map(np.atleast_1d, pm)
    shape = muacosd.shape

    # coordinates of NGP
    ag = coord.Galactic._ngp_J2000.ra
    dg = coord.Galactic._ngp_J2000.dec

    # used for the transformation matrix from Bovy (2011)
    cosphi = (np.sin(dg) - np.sin(i.dec)*np.sin(g.b)) / np.cos(g.b) / np.cos(i.dec)
    sinphi = np.sin(i.ra - ag) * np.cos(dg) / np.cos(g.b)

    R = np.zeros((2,2)+shape)
    R[0,0] = cosphi
    R[0,1] = sinphi
    R[1,0] = -sinphi
    R[1,1] = cosphi

    mu = np.vstack((muacosd.value[None], mud.to(muacosd.unit).value[None]))
    # new_mu = np.zeros_like(mu)
    # for i in range(n):
    #     new_mu[:,i] = R[...,i].dot(mu[:,i])
    new_mu = np.einsum('ijk...,jk...->ik...',R,mu)

    if coordinate.isscalar:
        return new_mu.reshape((2,))*muacosd.unit
    else:
        return new_mu.reshape((2,) + muacosd.shape)*muacosd.unit



cdef void sat_rotation_matrix(double *w, double *R)

cdef void to_sat_coords(double *w, double *R,
                        double *w_prime)

cdef void from_sat_coords(double *w_prime, double *R,
                          double *w)

cdef void car_to_cyl(double *w, double *cyl)
cdef void cyl_to_car(double *cyl, double *w)



# coding: utf-8

""" Velocity transformations for coordinate representations. """

from __future__ import division, print_function

__author__ = "adrn <adrn@astro.columbia.edu>"

# Standard library

# Third-party
import numpy as np
import astropy.coordinates as coord

__all__ = ['cartesian_to_spherical', 'cartesian_to_physicsspherical',
           'cartesian_to_cylindrical', 'spherical_to_cartesian',
           'physicsspherical_to_cartesian', 'cylindrical_to_cartesian']

def _pos_to_repr(pos):

    if hasattr(pos, 'xyz'):  # Representation-like
        pos_repr = coord.CartesianRepresentation(pos.xyz)

    elif hasattr(pos, 'cartesian') or hasattr(pos, 'to_cartesian'):  # Frame-like
        pos_repr = pos.represent_as(coord.CartesianRepresentation)

    elif hasattr(pos, 'unit'):  # Quantity-like
        pos_repr = coord.CartesianRepresentation(pos)

    else:
        raise TypeError("Unsupported position type '{0}'. Position must be "
                        "an Astropy Representation or Frame, or a "
                        "Quantity instance".format(type(pos)))

    return pos_repr

def cartesian_to_spherical(pos, vel):
    r"""
    Convert a velocity in Cartesian coordinates to velocity components
    in spherical coordinates. This follows the naming convention used in
    :mod:`astropy.coordinates`: spherical coordinates consist of a distance,
    :math:`d`, a longitude, :math:`\phi`, in the range [0, 360] deg, and
    a latitude, :math:`b`, in the range [-90, 90] deg.

    The components of the output velocity all have units of velocity, i.e.,
    this is not used for transforming from a cartesian velocity to angular
    velocities, but rather the velocity vector components in Eq. 2 below.

    .. math::

        \boldsymbol{v} &= v_x\boldsymbol{\hat{x}} + v_y\boldsymbol{\hat{y}} + v_z\boldsymbol{\hat{z}}\\\\
        &= v_r\boldsymbol{\hat{d}} + v_\phi\boldsymbol{\hat{\phi}} + v_b\boldsymbol{\hat{b}}\\\\
        &= \dot{d}\boldsymbol{\hat{d}} + d\cos b \dot{\phi}\boldsymbol{\hat{\phi}} + d\dot{b}\boldsymbol{\hat{b}}

    Parameters
    ----------
    pos : :class:`~astropy.units.Quantity`, :class:`~astropy.coordinates.BaseCoordinateFrame`, :class:`~astropy.coordinates.BaseRepresentation`
        Input position or positions as one of the allowed types. You may pass in a
        :class:`~astropy.units.Quantity` with :class:`~astropy.units.dimensionless_unscaled`
        units if you are working in natural units.
    vel : :class:`~astropy.units.Quantity`
        Input velocity or velocities as one of the allowed types. You may pass in a
        :class:`~astropy.units.Quantity` with :class:`~astropy.units.dimensionless_unscaled`
        units if you are working in natural units. axis=0 is assumed to be the
        dimensionality axis, e.g., ``vx,vy,vz = vel`` should work.

    Returns
    -------
    vsph : :class:`~astropy.units.Quantity`
        Array of spherical velocity components. Will have the same shape as the
        input velocity.

    Examples
    --------

    """

    # position in Cartesian and spherical
    car_pos = _pos_to_repr(pos)
    sph_pos = car_pos.represent_as(coord.SphericalRepresentation)

    if not hasattr(vel, 'unit'):
        raise TypeError("Unsupported velocity type '{}'. Velocity must be "
                        "an Astropy Quantity instance.".format(type(vel)))

    # get out spherical components
    d = sph_pos.distance
    dxy = np.sqrt(car_pos.x**2 + car_pos.y**2)

    vr = np.sum(car_pos.xyz * vel, axis=0) / d

    mu_lon = (car_pos.xyz[0]*vel[1] - vel[0]*car_pos.xyz[1]) / dxy**2
    vlon = mu_lon * d * np.cos(sph_pos.lat)

    mu_lat = (car_pos.xyz[2]*(car_pos.xyz[0]*vel[0] + car_pos.xyz[1]*vel[1]) - dxy**2*vel[2]) / d**2 / dxy
    vlat = -mu_lat * d

    vsph = np.zeros_like(vel)
    vsph[0] = vr
    vsph[1] = vlon
    vsph[2] = vlat
    return vsph

def cartesian_to_physicsspherical(pos, vel):
    r"""
    Convert a velocity in Cartesian coordinates to velocity components
    in spherical coordinates. This follows the naming convention used in
    :mod:`astropy.coordinates`: physics spherical coordinates consist of
    a radius, :math:`r`, a longitude, :math:`\phi`, in the range [0, 360]
    deg and a colatitude, :math:`\theta`, in the range [0, 180] deg,
    measured from the z-axis.

    The components of the output velocity all have units of velocity, i.e.,
    this is not used for transforming from a cartesian velocity to angular
    velocities, but rather the velocity vector components in Eq. 2 below.

    .. math::

        \boldsymbol{v} &= v_x\boldsymbol{\hat{x}} + v_y\boldsymbol{\hat{y}} + v_z\boldsymbol{\hat{z}}\\\\
        &= v_r\boldsymbol{\hat{r}} + v_\phi\boldsymbol{\hat{\phi}} + v_\theta\boldsymbol{\hat{\theta}}\\\\
        &= \dot{r}\boldsymbol{\hat{r}} + r\sin\theta \dot{\phi}\boldsymbol{\hat{\phi}} + r\dot{\theta}\boldsymbol{\hat{\theta}}

    Parameters
    ----------
    pos : :class:`~astropy.units.Quantity`, :class:`~astropy.coordinates.BaseCoordinateFrame`, :class:`~astropy.coordinates.BaseRepresentation`
        Input position or positions as one of the allowed types. You may pass in a
        :class:`~astropy.units.Quantity` with :class:`~astropy.units.dimensionless_unscaled`
        units if you are working in natural units.
    vel : :class:`~astropy.units.Quantity`
        Input velocity or velocities as one of the allowed types. You may pass in a
        :class:`~astropy.units.Quantity` with :class:`~astropy.units.dimensionless_unscaled`
        units if you are working in natural units. axis=0 is assumed to be the
        dimensionality axis, e.g., ``vx,vy,vz = vel`` should work.

    Returns
    -------
    vsph : :class:`~astropy.units.Quantity`
        Array of spherical velocity components. Will have the same shape as the
        input velocity.

    Examples
    --------

    """

    # position in Cartesian and spherical
    car_pos = _pos_to_repr(pos)
    sph_pos = car_pos.represent_as(coord.PhysicsSphericalRepresentation)

    if not hasattr(vel, 'unit'):
        raise TypeError("Unsupported velocity type '{}'. Velocity must be "
                        "an Astropy Quantity instance.".format(type(vel)))

    # get out spherical components
    r = sph_pos.r
    dxy = np.sqrt(car_pos.x**2 + car_pos.y**2)

    vr = np.sum(car_pos.xyz * vel, axis=0) / r

    mu_lon = (car_pos.xyz[0]*vel[1] - vel[0]*car_pos.xyz[1]) / dxy**2
    vlon = mu_lon * r * np.sin(sph_pos.theta)

    mu_lat = (car_pos.xyz[2]*(car_pos.xyz[0]*vel[0] + car_pos.xyz[1]*vel[1]) - dxy**2*vel[2]) / r**2 / dxy
    vlat = mu_lat * r

    vsph = np.zeros_like(vel)
    vsph[0] = vr
    vsph[1] = vlon
    vsph[2] = vlat
    return vsph

def cartesian_to_cylindrical(pos, vel):
    r"""
    Convert a velocity in Cartesian coordinates to velocity components
    in cylindrical coordinates. This follows the naming convention used in
    :mod:`astropy.coordinates`: cylindrical coordinates consist of a radius,
    :math:`\rho`, an azimuthal angle, :math:`\phi`, in the range [0, 360] deg,
    and a z coordinate, :math:`z`.

    The components of the output velocity all have units of velocity, i.e.,
    this is not used for transforming from a cartesian velocity to angular
    velocities, but rather the velocity vector components in Eq. 2 below.

    .. math::

        \boldsymbol{v} &= v_x\boldsymbol{\hat{x}} + v_y\boldsymbol{\hat{y}} + v_z\boldsymbol{\hat{z}}\\\\
        &= v_\rho\boldsymbol{\hat{\rho}} + v_\phi\boldsymbol{\hat{\phi}} + v_z\boldsymbol{\hat{z}}\\\\
        &= \dot{\rho}\boldsymbol{\hat{\rho}} + \rho\dot{\phi}\boldsymbol{\hat{\phi}} + \dot{z}\boldsymbol{\hat{\theta}}

    Parameters
    ----------
    pos : :class:`~astropy.units.Quantity`, :class:`~astropy.coordinates.BaseCoordinateFrame`, :class:`~astropy.coordinates.BaseRepresentation`
        Input position or positions as one of the allowed types. You may pass in a
        :class:`~astropy.units.Quantity` with :class:`~astropy.units.dimensionless_unscaled`
        units if you are working in natural units.
    vel : :class:`~astropy.units.Quantity`
        Input velocity or velocities as one of the allowed types. You may pass in a
        :class:`~astropy.units.Quantity` with :class:`~astropy.units.dimensionless_unscaled`
        units if you are working in natural units. axis=0 is assumed to be the
        dimensionality axis, e.g., ``vx,vy,vz = vel`` should work.

    Returns
    -------
    vcyl : :class:`~astropy.units.Quantity`
        Array of spherical velocity components. Will have the same shape as the
        input velocity.

    Examples
    --------

    """

    # position in Cartesian and spherical
    car_pos = _pos_to_repr(pos)
    cyl_pos = car_pos.represent_as(coord.CylindricalRepresentation)

    if not hasattr(vel, 'unit'):
        raise TypeError("Unsupported velocity type '{}'. Velocity must be "
                        "an Astropy Quantity instance.".format(type(vel)))

    # get out spherical components
    rho = cyl_pos.rho
    vrho = np.sum(car_pos.xyz[:2] * vel[:2], axis=0) / rho

    phidot = (car_pos.xyz[0]*vel[1] - vel[0]*car_pos.xyz[1]) / rho**2
    vphi = phidot * rho

    vcyl = np.zeros_like(vel)
    vcyl[0] = vrho
    vcyl[1] = vphi
    vcyl[2] = vel[2]
    return vcyl

def spherical_to_cartesian(pos, vel):
    r"""
    Convert a velocity in Spherical coordinates to Cartesian coordinates.
    This follows the naming convention used in :mod:`astropy.coordinates`:
    spherical coordinates consist of a distance, :math:`d`, a longitude,
    :math:`\phi`, in the range [0, 360] deg, and a latitude, :math:`b`,
    in the range [-90, 90] deg.

    All components of the input spherical velocity should have units of
    velocity, i.e., this is not used for transforming angular velocities,
    but rather the velocity vector components in Eq. 2 below.

    .. math::

        \boldsymbol{v} &= v_x\boldsymbol{\hat{x}} + v_y\boldsymbol{\hat{y}} + v_z\boldsymbol{\hat{z}}\\\\
        &= v_r\boldsymbol{\hat{d}} + v_\phi\boldsymbol{\hat{\phi}} + v_b\boldsymbol{\hat{b}}\\\\
        &= \dot{d}\boldsymbol{\hat{d}} + d\cos b \dot{\phi}\boldsymbol{\hat{\phi}} + d\dot{b}\boldsymbol{\hat{b}}

    Parameters
    ----------
    pos : :class:`~astropy.units.Quantity`, :class:`~astropy.coordinates.BaseCoordinateFrame`, :class:`~astropy.coordinates.BaseRepresentation`
        Input position or positions as one of the allowed types. You may pass in a
        :class:`~astropy.units.Quantity` with :class:`~astropy.units.dimensionless_unscaled`
        units if you are working in natural units.
    vel : :class:`~astropy.units.Quantity`
        Input velocity or velocities as one of the allowed types. You may pass in a
        :class:`~astropy.units.Quantity` with :class:`~astropy.units.dimensionless_unscaled`
        units if you are working in natural units. axis=0 is assumed to be the
        dimensionality axis, e.g., ``vx,vy,vz = vel`` should work.

    Returns
    -------
    vxyz : :class:`~astropy.units.Quantity`
        Array of Cartesian velocity components. Will have the same shape as the
        input velocity.

    Examples
    --------

    """

    # position in Cartesian and spherical
    car_pos = _pos_to_repr(pos)
    sph_pos = car_pos.represent_as(coord.SphericalRepresentation)

    if not hasattr(vel, 'unit'):
        raise TypeError("Unsupported velocity type '{}'. Velocity must be "
                        "an Astropy Quantity instance.".format(type(vel)))

    # get out spherical components
    phi = sph_pos.lon
    lat = sph_pos.lat
    vr,vlon,vlat = vel

    vxyz = np.zeros_like(vel)
    vxyz[0] = vr*np.cos(phi)*np.cos(lat) - np.sin(phi)*vlon - np.cos(phi)*np.sin(lat)*vlat
    vxyz[1] = vr*np.sin(phi)*np.cos(lat) + np.cos(phi)*vlon - np.sin(phi)*np.sin(lat)*vlat
    vxyz[2] = vr*np.sin(lat) + np.cos(lat)*vlat
    return vxyz

def physicsspherical_to_cartesian(pos, vel):
    r"""
    Convert a velocity in Spherical coordinates to Cartesian coordinates.
    This follows the naming convention used in :mod:`astropy.coordinates`:
    physics spherical coordinates consist of a radius, :math:`r`, a longitude,
    :math:`\phi`, in the range [0, 360] deg and a colatitude, :math:`\theta`,
    in the range [0, 180] deg, measured from the z-axis.

    The components of the output velocity all have units of velocity, i.e.,
    this is not used for transforming from a cartesian velocity to angular
    velocities, but rather the velocity vector components in Eq. 2 below.

    .. math::

        \boldsymbol{v} &= v_x\boldsymbol{\hat{x}} + v_y\boldsymbol{\hat{y}} + v_z\boldsymbol{\hat{z}}\\\\
        &= v_r\boldsymbol{\hat{r}} + v_\phi\boldsymbol{\hat{\phi}} + v_\theta\boldsymbol{\hat{\theta}}\\\\
        &= \dot{r}\boldsymbol{\hat{r}} + r\sin\theta \dot{\phi}\boldsymbol{\hat{\phi}} + r\dot{\theta}\boldsymbol{\hat{\theta}}

    Parameters
    ----------
    pos : :class:`~astropy.units.Quantity`, :class:`~astropy.coordinates.BaseCoordinateFrame`, :class:`~astropy.coordinates.BaseRepresentation`
        Input position or positions as one of the allowed types. You may pass in a
        :class:`~astropy.units.Quantity` with :class:`~astropy.units.dimensionless_unscaled`
        units if you are working in natural units.
    vel : :class:`~astropy.units.Quantity`
        Input velocity or velocities as one of the allowed types. You may pass in a
        :class:`~astropy.units.Quantity` with :class:`~astropy.units.dimensionless_unscaled`
        units if you are working in natural units. axis=0 is assumed to be the
        dimensionality axis, e.g., ``vx,vy,vz = vel`` should work.

    Returns
    -------
    vxyz : :class:`~astropy.units.Quantity`
        Array of Cartesian velocity components. Will have the same shape as the
        input velocity.

    Examples
    --------

    """

    # position in Cartesian and spherical
    car_pos = _pos_to_repr(pos)
    sph_pos = car_pos.represent_as(coord.PhysicsSphericalRepresentation)

    if not hasattr(vel, 'unit'):
        raise TypeError("Unsupported velocity type '{}'. Velocity must be "
                        "an Astropy Quantity instance.".format(type(vel)))

    # get out spherical components
    phi = sph_pos.phi
    the = sph_pos.theta
    vr,vphi,vthe = vel
    vthe = -vthe

    vxyz = np.zeros_like(vel)
    vxyz[0] = vr*np.cos(phi)*np.sin(the) - np.sin(phi)*vphi - np.cos(phi)*np.cos(the)*vthe
    vxyz[1] = vr*np.sin(phi)*np.sin(the) + np.cos(phi)*vphi - np.sin(phi)*np.cos(the)*vthe
    vxyz[2] = vr*np.cos(the) + np.sin(the)*vthe
    return vxyz

def cylindrical_to_cartesian(pos, vel):
    r"""
    Convert a velocity in Spherical coordinates to Cylindrical coordinates.
    This follows the naming convention used in :mod:`astropy.coordinates`:
    cylindrical coordinates consist of a radius, :math:`\rho`, an azimuthal angle,
    :math:`\phi`, in the range [0, 360] deg, and a z coordinate, :math:`z`.

    The components of the output velocity all have units of velocity, i.e.,
    this is not used for transforming from a cartesian velocity to angular
    velocities, but rather the velocity vector components in Eq. 2 below.

    .. math::

        \boldsymbol{v} &= v_x\boldsymbol{\hat{x}} + v_y\boldsymbol{\hat{y}} + v_z\boldsymbol{\hat{z}}\\\\
        &= v_\rho\boldsymbol{\hat{\rho}} + v_\phi\boldsymbol{\hat{\phi}} + v_z\boldsymbol{\hat{z}}\\\\
        &= \dot{\rho}\boldsymbol{\hat{\rho}} + \rho\dot{\phi}\boldsymbol{\hat{\phi}} + \dot{z}\boldsymbol{\hat{\theta}}

    Parameters
    ----------
    pos : :class:`~astropy.units.Quantity`, :class:`~astropy.coordinates.BaseCoordinateFrame`, :class:`~astropy.coordinates.BaseRepresentation`
        Input position or positions as one of the allowed types. You may pass in a
        :class:`~astropy.units.Quantity` with :class:`~astropy.units.dimensionless_unscaled`
        units if you are working in natural units.
    vel : :class:`~astropy.units.Quantity`
        Input velocity or velocities as one of the allowed types. You may pass in a
        :class:`~astropy.units.Quantity` with :class:`~astropy.units.dimensionless_unscaled`
        units if you are working in natural units. axis=0 is assumed to be the
        dimensionality axis, e.g., ``vx,vy,vz = vel`` should work.

    Returns
    -------
    vxyz : :class:`~astropy.units.Quantity`
        Array of Cartesian velocity components. Will have the same shape as the
        input velocity.

    Examples
    --------

    """

    # position in Cartesian and cylindrical
    car_pos = _pos_to_repr(pos)
    cyl_pos = car_pos.represent_as(coord.CylindricalRepresentation)

    if not hasattr(vel, 'unit'):
        raise TypeError("Unsupported velocity type '{}'. Velocity must be "
                        "an Astropy Quantity instance.".format(type(vel)))

    # get out spherical components
    phi = cyl_pos.phi
    vrho,vphi,vz = vel

    vcyl = np.zeros_like(vel)
    vcyl[0] = vrho * np.cos(phi) - np.sin(phi) * vphi
    vcyl[1] = vrho * np.sin(phi) + np.cos(phi) * vphi
    vcyl[2] = vz
    return vcyl



# coding: utf-8

from __future__ import division, print_function

__author__ = "adrn <adrn@astro.columbia.edu>"

# Third-party
from astropy import log as logger
import astropy.units as u
uno = u.dimensionless_unscaled
import numpy as np
from scipy.signal import argrelmin, argrelmax

# Project
from .core import CartesianPhaseSpacePosition
from .util import peak_to_peak_period
from .plot import plot_orbits
from ..util import atleast_2d

__all__ = ['CartesianOrbit', 'combine']

class Orbit(object):

    # ------------------------------------------------------------------------
    # Shape and size
    # ------------------------------------------------------------------------
    @property
    def ntimes(self):
        return self.pos.shape[1]

    @property
    def norbits(self):
        if self.pos.ndim < 3:
            return 1
        else:
            return self.pos.shape[2]

    # ------------------------------------------------------------------------
    # Computed dynamical quantities
    # ------------------------------------------------------------------------
    def pericenter(self, type=np.mean):
        """
        Estimate the pericenter(s) of the orbit. By default, this returns
        the mean pericenter. To get, e.g., the minimum pericenter,
        pass in ``type=np.min``. To get all pericenters, pass in
        ``type=None``.

        Parameters
        ----------
        type : func (optional)
            By default, this returns the mean pericenter. To return all
            pericenters, pass in ``None``. To get, e.g., the minimum
            or maximum pericenter, pass in ``np.min`` or ``np.max``.

        Returns
        -------
        peri : float, :class:`~numpy.ndarray`
            Either a single number or an array of pericenters.
        """
        r = self.r
        min_ix = argrelmin(r, mode='wrap')[0]
        min_ix = min_ix[(min_ix != 0) & (min_ix != (len(r)-1))]

        if type is not None:
            return type(r[min_ix])
        else:
            return r[min_ix]

    def apocenter(self, type=np.mean):
        """
        Estimate the apocenter(s) of the orbit. By default, this returns
        the mean apocenter. To get, e.g., the minimum apocenter,
        pass in ``type=np.min``. To get all apocenters, pass in
        ``type=None``.

        Parameters
        ----------
        type : func (optional)
            By default, this returns the mean apocenter. To return all
            apocenters, pass in ``None``. To get, e.g., the minimum
            or maximum apocenter, pass in ``np.min`` or ``np.max``.

        Returns
        -------
        apo : float, :class:`~numpy.ndarray`
            Either a single number or an array of apocenters.
        """
        r = self.r
        max_ix = argrelmax(r, mode='wrap')[0]
        max_ix = max_ix[(max_ix != 0) & (max_ix != (len(r)-1))]

        if type is not None:
            return type(r[max_ix])
        else:
            return r[max_ix]

    def eccentricity(self):
        r"""
        Returns the eccentricity computed from the mean apocenter and
        mean pericenter.

        .. math::

            e = \frac{r_{\rm apo} - r_{\rm per}}{r_{\rm apo} + r_{\rm per}}

        Returns
        -------
        ecc : float
            The orbital eccentricity.

        """
        ra = self.apocenter()
        rp = self.pericenter()
        return (ra - rp) / (ra + rp)

    def estimate_period(self, radial=True):
        """
        Estimate the period of the orbit. By default, computes the radial
        period. If ``radial==False``, this returns period estimates for
        each dimension of the orbit.

        Parameters
        ----------
        radial : bool (optional)
            What period to estimate. If ``True``, estimates the radial
            period. If ``False``, estimates period in each dimension, e.g.,
            if the orbit is 3D, along x, y, and z.

        Returns
        -------
        T : `~astropy.units.Quantity`
            The period or periods.
        """

        if self.t is None:
            raise ValueError("To compute the period, a time array is needed."
                             " Specify a time array when creating this object.")

        if radial:
            r = self.r.value
            if self.norbits == 1:
                T = peak_to_peak_period(self.t.value, r)
                T = T * self.t.unit
            else:
                T = [peak_to_peak_period(self.t.value, r[:,n]) for n in range(r.shape[1])]
                T = T * self.t.unit

        else:
            raise NotImplementedError("sorry 'bout that...")

        return T

class CartesianOrbit(CartesianPhaseSpacePosition, Orbit):
    """
    Represents an orbit in Cartesian coordinates -- positions
    and velocities (conjugate momenta) at different times.

    .. warning::

        This is an experimental class. The API may change in a future release!

    The position and velocity quantities (arrays) can have an arbitrary
    number of dimensions, but the first two axes (0, 1) have
    special meaning::

        - `axis=0` is the coordinate dimension (e.g., x, y, z)
        - `axis=1` is the time dimension

    So if the input position array, `pos`, has shape `pos.shape = (3, 100)`, this
    would be a 3D orbit (`pos[0]` is `x`, `pos[1]` is `y`, etc.). For representing
    multiple orbits, the position array could have 3 axes, e.g., it might have shape
    `pos.shape = (3, 100, 8)`, where this is interpreted as a 3D position at 100 times
    for 8 different orbits. The same is true for velocity. The position and velocity
    arrays must have the same shape.

    If a time argument is specified, the position and velocity arrays must have
    the same number of timesteps as the length of the time object::

        len(t) == pos.shape[1]

    Parameters
    ----------
    pos : :class:`~astropy.units.Quantity`, array_like
        Positions. If a numpy array (e.g., has no units), this will be
        stored as a dimensionless :class:`~astropy.units.Quantity`. See
        the note above about the assumed meaning of the axes of this object.
    vel : :class:`~astropy.units.Quantity`, array_like
        Velocities. If a numpy array (e.g., has no units), this will be
        stored as a dimensionless :class:`~astropy.units.Quantity`. See
        the note above about the assumed meaning of the axes of this object.
    t : array_like, :class:`~astropy.units.Quantity` (optional)
        Array of times. If a numpy array (e.g., has no units), this will be
        stored as a dimensionless :class:`~astropy.units.Quantity`.
    potential : `~gala.potential.PotentialBase` (optional)
        The potential that the orbit was integrated in.

    """
    def __init__(self, pos, vel, t=None, potential=None):

        super(CartesianOrbit, self).__init__(pos=pos, vel=vel)

        if t is not None:
            t = np.atleast_1d(t)
            if self.pos.shape[1] != len(t):
                raise ValueError("Position and velocity must have the same length "
                                 "along axis=1 as the length of the time array "
                                 "{} vs {}".format(len(t), self.pos.shape[1]))

            if not hasattr(t, 'unit'):
                t = t * uno

        self.t = t
        self.potential = potential

    def __getitem__(self, slyce):
        if isinstance(slyce, np.ndarray) or isinstance(slyce, list):
            _slyce = np.array(slyce)
            _slyce = (slice(None),) + (slyce,)
        else:
            try:
                _slyce = (slice(None),) + tuple(slyce)
            except TypeError:
                _slyce = (slice(None),) + (slyce,)

        kw = dict()
        if self.t is not None:
            kw['t'] = self.t[_slyce[1]]

        pos = self.pos[_slyce]
        vel = self.vel[_slyce]

        if isinstance(_slyce[1], int) or pos.ndim == 1:
            return CartesianPhaseSpacePosition(pos=pos, vel=vel)
        else:
            return self.__class__(pos=pos, vel=vel,
                                  potential=self.potential, **kw)

    def w(self, units=None):
        """
        This returns a single array containing the phase-space positions.

        Parameters
        ----------
        units : `~gala.units.UnitSystem` (optional)
            The unit system to represent the position and velocity in
            before combining into the full array.

        Returns
        -------
        w : `~numpy.ndarray`
            A numpy array of all positions and velocities, without units.
            Will have shape ``(2*ndim,...)``.

        """
        if self.pos.unit == uno and self.vel.unit == uno and units is None:
            units = [uno]

        else:
            if units is None and self.potential is None:
                raise ValueError("If no UnitSystem is specified, the orbit must have "
                                 "an associated potential object.")

            if units is None and self.potential.units is None:
                raise ValueError("If no UnitSystem is specified, the potential object "
                                 "associated with this orbit must have a UnitSystem defined.")

            if units is None:
                units = self.potential.units

        x = self.pos.decompose(units).value
        v = self.vel.decompose(units).value
        w = np.vstack((x,v))
        if w.ndim < 3:
            w = w[...,np.newaxis] # one orbit
        return w

    # ------------------------------------------------------------------------
    # Computed dynamical quantities
    # ------------------------------------------------------------------------

    @property
    def r(self):
        """
        The orbital radius.
        """
        return np.sqrt(np.sum(self.pos**2, axis=0))

    def potential_energy(self, potential=None):
        r"""
        The potential energy *per unit mass*:

        .. math::

            E_\Phi = \Phi(\boldsymbol{q})

        Returns
        -------
        E : :class:`~astropy.units.Quantity`
            The potential energy.
        """
        if self.potential is None and potential is None:
            raise ValueError("To compute the potential energy, a potential"
                             " object must be provided!")
        if potential is None:
            potential = self.potential

        return super(CartesianOrbit,self).potential_energy(potential)

    def energy(self, potential=None):
        r"""
        The total energy *per unit mass* (e.g., kinetic + potential):

        Returns
        -------
        E : :class:`~astropy.units.Quantity`
            The total energy.
        """
        return self.kinetic_energy() + self.potential_energy(potential)

    # ------------------------------------------------------------------------
    # Misc. useful methods
    # ------------------------------------------------------------------------
    def circulation(self):
        """
        Determine which axes the Orbit circulates around by checking
        whether there is a change of sign of the angular momentum
        about an axis. Returns a 2D array with ``ndim`` integers per orbit
        point. If a box orbit, all integers will be 0. A 1 indicates
        circulation about the corresponding axis.

        TODO: clockwise / counterclockwise?

        For example, for a single 3D orbit:

        - Box and boxlet = [0,0,0]
        - z-axis (short-axis) tube = [0,0,1]
        - x-axis (long-axis) tube = [1,0,0]

        Returns
        -------
        circulation : :class:`numpy.ndarray`
            An array that specifies whether there is circulation about any of
            the axes of the input orbit. For a single orbit, will return a
            1D array, but for multiple orbits, the shape will be ``(3, norbits)``.

        """
        L = self.angular_momentum()

        # if only 2D, add another empty axis
        if L.ndim == 2:
            single_orbit = True
            L = L[...,None]
        else:
            single_orbit = False

        ndim,ntimes,norbits = L.shape

        # initial angular momentum
        L0 = L[:,0]

        # see if at any timestep the sign has changed
        circ = np.ones((ndim,norbits))
        for ii in range(ndim):
            cnd = (np.sign(L0[ii]) != np.sign(L[ii,1:])) | \
                  (np.abs(L[ii,1:]).value < 1E-13)
            ix = np.atleast_1d(np.any(cnd, axis=0))
            circ[ii,ix] = 0

        circ = circ.astype(int)
        if single_orbit:
            return circ.reshape((ndim,))
        else:
            return circ

    def align_circulation_with_z(self, circulation=None):
        """
        If the input orbit is a tube orbit, this function aligns the circulation
        axis with the z axis and returns a copy.

        Parameters
        ----------
        circulation : array_like (optional)
            Array of bits that specify the axis about which the orbit circulates. If
            not provided, will compute this using
            :meth:`~gala.dynamics.CartesianOrbit.circulation`. See that method for
            more information.

        Returns
        -------
        orb : :class:`~gala.dynamics.CartesianOrbit`
            A copy of the original orbit object with circulation aligned with the z axis.
        """

        if circulation is None:
            circulation = self.circulation()
        circulation = atleast_2d(circulation, insert_axis=1)

        if self.pos.ndim < 3:
            pos = self.pos[...,np.newaxis]
            vel = self.vel[...,np.newaxis]
        else:
            pos = self.pos
            vel = self.vel

        if circulation.shape[0] != self.ndim or circulation.shape[1] != pos.shape[2]:
            raise ValueError("Shape of 'circulation' array should match the shape"
                             " of the position/velocity (minus the time axis).")

        new_pos = pos.copy()
        new_vel = vel.copy()
        for n in range(pos.shape[2]):
            if circulation[2,n] == 1 or np.all(circulation[:,n] == 0):
                # already circulating about z or box orbit
                continue

            if sum(circulation[:,n]) > 1:
                logger.warning("Circulation about multiple axes - are you sure "
                               "the orbit has been integrated for long enough?")

            if circulation[0,n] == 1:
                circ = 0
            elif circulation[1,n] == 1:
                circ = 1
            else:
                raise RuntimeError("Should never get here...")

            new_pos[circ,:,n] = pos[2,:,n]
            new_pos[2,:,n] = pos[circ,:,n]

            new_vel[circ,:,n] = vel[2,:,n]
            new_vel[2,:,n] = vel[circ,:,n]

        return self.__class__(pos=new_pos, vel=new_vel, t=self.t, potential=self.potential)

    def plot(self, **kwargs):
        """
        Plot the orbit in all projections. This is a thin wrapper around
        `~gala.dynamics.plot_orbits` -- the docstring for this function is
        included here.

        .. warning::

            This will currently fail for orbits with fewer than 3 dimensions.

        Parameters
        ----------
        ix : int, array_like (optional)
            Index or array of indices of orbits to plot. For example, if `x` is an
            array of shape ``(3,1024,32)`` - 1024 timesteps for 32 orbits in 3D
            positions -- `ix` would specify which of the 32 orbits to plot.
        axes : array_like (optional)
            Array of matplotlib Axes objects.
        triangle : bool (optional)
            Make a triangle plot instead of plotting all projections in a single row.
        subplots_kwargs : dict (optional)
            Dictionary of kwargs passed to :func:`~matplotlib.pyplot.subplots`.
        labels : iterable (optional)
            List or iterable of axis labels as strings. They should correspond to the
            dimensions of the input orbit.
        **kwargs
            All other keyword arguments are passed to :func:`~matplotlib.pyplot.plot`.
            You can pass in any of the usual style kwargs like ``color=...``,
            ``marker=...``, etc.

        Returns
        -------
        fig : `~matplotlib.Figure`

        """
        _label_unit = ''
        if self.pos.unit != u.dimensionless_unscaled:
            _label_unit = ' [{}]'.format(self.pos.unit)

        default_kwargs = {
            'marker': None,
            'linestyle': '-',
            'labels': ('$x${}'.format(_label_unit),
                       '$y${}'.format(_label_unit),
                       '$z${}'.format(_label_unit))
        }

        for k,v in default_kwargs.items():
            kwargs[k] = kwargs.get(k, v)

        return plot_orbits(self.pos.value, **kwargs)

def combine(args, along_time_axis=False):
    """
    Combine the input `Orbit` objects into a single object.

    The `Orbits` must all have the same potential and time array.

    Parameters
    ----------
    args : iterable
        Multiple instances of `Orbit` objects.
    along_time_axis : bool (optional)
        If True, will combine single orbits along the time axis.

    Returns
    -------
    obj : orbit_like
        A single objct with positions and velocities stacked along the last axis.

    """

    ndim = None
    time = None
    pot = None
    pos_unit = None
    vel_unit = None
    cls = None

    all_pos = []
    all_vel = []
    all_time = []
    for x in args:
        if ndim is None:
            ndim = x.ndim
            pos_unit = x.pos.unit
            vel_unit = x.vel.unit
            time = x.t
            if time is not None:
                t_unit = time.unit
            else:
                t_unit = None
            pot = x.potential
            cls = x.__class__
        else:
            if x.__class__.__name__ != cls.__name__:
                raise ValueError("All objects must have the same class.")

            if x.ndim != ndim:
                raise ValueError("All objects must have the same dimensionality.")

            if not along_time_axis:
                if time is not None:
                    if x.t is None or len(x.t) != len(time) or np.any(x.t.to(time.unit).value != time.value):
                        raise ValueError("All orbits must have the same time array.")

            if x.potential != pot:
                raise ValueError("All orbits must have the same Potential object.")

        pos = x.pos
        if pos.ndim < 3:
            pos = pos[...,np.newaxis]

        vel = x.vel
        if vel.ndim < 3:
            vel = vel[...,np.newaxis]

        all_pos.append(pos.to(pos_unit).value)
        all_vel.append(vel.to(vel_unit).value)
        if time is not None:
            all_time.append(x.t.to(t_unit).value)

    norbits = np.array([pos.shape[-1] for pos in all_pos] + [pos.shape[-1] for pos in all_vel])
    if along_time_axis:
        if np.all(norbits == norbits[0]):
            all_pos = np.hstack(all_pos)*pos_unit
            all_vel = np.hstack(all_vel)*vel_unit
            if len(all_time) > 0:
                all_time = np.concatenate(all_time)*t_unit
            else:
                all_time = None
        else:
            raise ValueError("To combine along time axis, all orbit objects must have "
                             "the same number of orbits.")
        if args[0].pos.ndim == 2:
            all_pos = all_pos[...,0]
            all_vel = all_vel[...,0]

    else:
        all_pos = np.dstack(all_pos)*pos_unit
        all_vel = np.dstack(all_vel)*vel_unit
        if len(all_time) > 0:
            all_time = all_time[0]*t_unit
        else:
            all_time = None

    return cls(pos=all_pos, vel=all_vel, t=all_time, potential=args[0].potential)



# coding: utf-8

""" Helper function for turning different ways of specifying the integration
    times into an array of times.
"""

from __future__ import division, print_function

__author__ = "adrn <adrn@astro.columbia.edu>"

# Standard library
from warnings import warn

# Third-party
import numpy as np

__all__ = ['parse_time_specification']

def parse_time_specification(units, dt=None, n_steps=None, nsteps=None, t1=None, t2=None, t=None):
    """
    Return an array of times given a few combinations of kwargs that are
    accepted -- see below.

    Parameters
    ----------
    dt, n_steps[, t1] : (numeric, int[, numeric])
        A fixed timestep dt and a number of steps to run for.
    dt, t1, t2 : (numeric, numeric, numeric)
        A fixed timestep dt, an initial time, and an final time.
    dt, t1 : (array_like, numeric)
        An array of timesteps dt and an initial time.
    n_steps, t1, t2 : (int, numeric, numeric)
        Number of steps between an initial time, and a final time.
    t : array_like
        An array of times (dts = t[1:] - t[:-1])

    """
    if nsteps is not None:
        warn("The argument 'nsteps' is deprecated and will be removed in a future version."
             "Use 'n_steps' instead.")
        n_steps = nsteps

    if hasattr(dt, 'unit'):
        dt = dt.decompose(units).value

    if hasattr(t1, 'unit'):
        t1 = t1.decompose(units).value

    if hasattr(t2, 'unit'):
        t2 = t2.decompose(units).value

    if hasattr(t, 'unit'):
        t = t.decompose(units).value

    # t : array_like
    if t is not None:
        times = t
        return times

    else:
        if dt is None and (t1 is None or t2 is None or n_steps is None):
            raise ValueError("Invalid spec. See docstring.")

        # dt, n_steps[, t1] : (numeric, int[, numeric])
        elif dt is not None and n_steps is not None:
            if t1 is None:
                t1 = 0.

            times = parse_time_specification(units, dt=np.ones(n_steps+1)*dt, t1=t1)
        # dt, t1, t2 : (numeric, numeric, numeric)
        elif dt is not None and t1 is not None and t2 is not None:
            if t2 < t1 and dt < 0:

                t_i = t1
                times = []
                ii = 0
                while (t_i > t2) and (ii < 1E6):
                    times.append(t_i)
                    t_i += dt

                if times[-1] != t2:
                    times.append(t2)

                return np.array(times)

            elif t2 > t1 and dt > 0:

                t_i = t1
                times = []
                ii = 0
                while (t_i < t2) and (ii < 1E6):
                    times.append(t_i)
                    t_i += dt

                return np.array(times)

            else:
                raise ValueError("If t2 < t1, dt must be negative. If t1 < t2, "
                                 "dt should be positive.")

        # dt, t1 : (array_like, numeric)
        elif isinstance(dt, np.ndarray) and t1 is not None:
            times = np.cumsum(np.append([0.], dt)) + t1
            times = times[:-1]

        # n_steps, t1, t2 : (int, numeric, numeric)
        elif dt is None and not (t1 is None or t2 is None or n_steps is None):
            times = np.linspace(t1, t2, n_steps, endpoint=True)

        else:
            raise ValueError("Invalid options. See docstring.")

        return times



# coding: utf-8

""" Test core dynamics.  """

from __future__ import division, print_function

__author__ = "adrn <adrn@astro.columbia.edu>"

# Third-party
import astropy.units as u
from astropy.coordinates import SphericalRepresentation, Galactic
import numpy as np
import pytest

# Project
from ..core import *
from ...potential import HernquistPotential
from ...util import assert_quantities_allclose
from ...units import galactic, solarsystem

def test_initialize():

    with pytest.raises(ValueError):
        x = np.random.random(size=(3,10))
        v = np.random.random(size=(3,8))
        CartesianPhaseSpacePosition(pos=x, vel=v)

    x = np.random.random(size=(3,10))
    v = np.random.random(size=(3,10))
    o = CartesianPhaseSpacePosition(pos=x, vel=v)
    assert o.ndim == 3

    x = np.random.random(size=(3,10))*u.kpc
    v = np.random.random(size=(3,10))*u.km/u.s
    o = CartesianPhaseSpacePosition(pos=x, vel=v)
    assert o.pos.unit == u.kpc
    assert o.vel.unit == u.km/u.s

    x = np.random.random(size=(2,10))
    v = np.random.random(size=(2,10))
    o = CartesianPhaseSpacePosition(pos=x, vel=v)
    assert o.ndim == 2

def test_from_w():

    w = np.random.random(size=(6,10))
    o = CartesianPhaseSpacePosition.from_w(w, galactic)
    assert o.pos.unit == u.kpc
    assert o.vel.unit == u.kpc/u.Myr

def test_slice():

    # simple
    x = np.random.random(size=(3,10))
    v = np.random.random(size=(3,10))
    o = CartesianPhaseSpacePosition(pos=x, vel=v)
    new_o = o[:5]
    assert new_o.pos.shape == (3,5)
    assert new_o.vel.shape == (3,5)

    # 1d slice on 3d
    x = np.random.random(size=(3,100,8))
    v = np.random.random(size=(3,100,8))
    o = CartesianPhaseSpacePosition(pos=x, vel=v)
    new_o = o[:5]
    assert new_o.pos.shape == (3,5,8)
    assert new_o.vel.shape == (3,5,8)

    # 3d slice on 3d
    o = CartesianPhaseSpacePosition(pos=x, vel=v)
    new_o = o[:5,:4]
    assert new_o.pos.shape == (3,5,4)
    assert new_o.vel.shape == (3,5,4)

    # boolean array
    x = np.random.random(size=(3,10))
    v = np.random.random(size=(3,10))
    o = CartesianPhaseSpacePosition(pos=x, vel=v)
    ix = np.array([0,0,0,0,0,1,1,1,1,1]).astype(bool)
    new_o = o[ix]
    assert new_o.shape == (sum(ix),)

    # integer array
    x = np.random.random(size=(3,10))
    v = np.random.random(size=(3,10))
    o = CartesianPhaseSpacePosition(pos=x, vel=v)
    ix = np.array([0,3,5])
    new_o = o[ix]
    assert new_o.shape == (len(ix),)

# ------------------------------------------------------------------------
# Convert from Cartesian to other representations
# ------------------------------------------------------------------------
def test_represent_as():

    # simple / unitless
    x = np.random.random(size=(3,10))
    v = np.random.random(size=(3,10))
    o = CartesianPhaseSpacePosition(pos=x, vel=v)
    sph_pos, sph_vel = o.represent_as(SphericalRepresentation)

    assert sph_pos.distance.unit == u.dimensionless_unscaled
    assert sph_vel.unit == u.dimensionless_unscaled

    # simple / with units
    x = np.random.random(size=(3,10))*u.kpc
    v = np.random.normal(0.,100.,size=(3,10))*u.km/u.s
    o = CartesianPhaseSpacePosition(pos=x, vel=v)
    sph_pos, sph_vel = o.represent_as(SphericalRepresentation)
    assert sph_pos.distance.unit == u.kpc

def test_to_frame():
    # simple / unitless
    x = np.random.random(size=(3,10))
    v = np.random.random(size=(3,10))
    o = CartesianPhaseSpacePosition(pos=x, vel=v)

    with pytest.raises(u.UnitConversionError):
        o.to_frame(Galactic)

    # simple / with units
    x = np.random.random(size=(3,10))*u.kpc
    v = np.random.normal(0.,100.,size=(3,10))*u.km/u.s
    o = CartesianPhaseSpacePosition(pos=x, vel=v)
    coo,vel = o.to_frame(Galactic)
    assert coo.name == 'galactic'

def test_w():
    # simple / unitless
    x = np.random.random(size=(3,10))
    v = np.random.random(size=(3,10))
    o = CartesianPhaseSpacePosition(pos=x, vel=v)
    w = o.w()
    assert w.shape == (6,10)

    # simple / with units
    x = np.random.random(size=(3,10))*u.kpc
    v = np.random.normal(0.,100.,size=(3,10))*u.km/u.s
    o = CartesianPhaseSpacePosition(pos=x, vel=v)
    with pytest.raises(ValueError):
        o.w()
    w = o.w(units=galactic)
    assert np.allclose(x.value, w[:3])
    assert np.allclose(v.value, (w[3:]*u.kpc/u.Myr).to(u.km/u.s).value)

    # simple / with units and potential
    p = HernquistPotential(units=galactic, m=1E11, c=0.25)
    x = np.random.random(size=(3,10))*u.kpc
    v = np.random.normal(0.,100.,size=(3,10))*u.km/u.s
    o = CartesianPhaseSpacePosition(pos=x, vel=v)
    w = o.w(p.units)
    assert np.allclose(x.value, w[:3])
    assert np.allclose(v.value, (w[3:]*u.kpc/u.Myr).to(u.km/u.s).value)

    w = o.w(units=solarsystem)
    assert np.allclose(x.value, (w[:3]*u.au).to(u.kpc).value)
    assert np.allclose(v.value, (w[3:]*u.au/u.yr).to(u.km/u.s).value)

# ------------------------------------------------------------------------
# Computed dynamical quantities
# ------------------------------------------------------------------------
def test_energy():
    # with units
    x = np.random.random(size=(3,10))*u.kpc
    v = np.random.normal(0.,100.,size=(3,10))*u.km/u.s
    o = CartesianPhaseSpacePosition(pos=x, vel=v)
    KE = o.kinetic_energy()
    assert KE.unit == (o.vel.unit)**2
    assert KE.shape == o.pos.shape[1:]

    # with units and potential
    p = HernquistPotential(units=galactic, m=1E11, c=0.25)
    x = np.random.random(size=(3,10))*u.kpc
    v = np.random.normal(0.,100.,size=(3,10))*u.km/u.s
    o = CartesianPhaseSpacePosition(pos=x, vel=v)
    PE = o.potential_energy(potential=p)
    E = o.energy(potential=p)

def test_angular_momentum():

    w = CartesianPhaseSpacePosition([1.,0.,0.], [0.,0.,1.])
    assert np.allclose(np.squeeze(w.angular_momentum()), [0., -1, 0])

    w = CartesianPhaseSpacePosition([1.,0.,0.], [0.,1.,0.])
    assert np.allclose(np.squeeze(w.angular_momentum()), [0., 0, 1])

    w = CartesianPhaseSpacePosition([0.,1.,0.],[0.,0.,1.])
    assert np.allclose(np.squeeze(w.angular_momentum()), [1., 0, 0])

    w = CartesianPhaseSpacePosition([1.,0,0]*u.kpc, [0.,200.,0]*u.pc/u.Myr)
    assert_quantities_allclose(np.squeeze(w.angular_momentum()), [0,0,0.2]*u.kpc**2/u.Myr)

    # multiple - known
    q = np.array([[1.,0.,0.],[1.,0.,0.],[0,1.,0.]]).T
    p = np.array([[0,0,1.],[0,1.,0.],[0,0,1]]).T
    L = CartesianPhaseSpacePosition(q, p).angular_momentum()
    true_L = np.array([[0., -1, 0],[0., 0, 1],[1., 0, 0]]).T
    assert L.shape == (3,3)
    assert np.allclose(L, true_L)

    # multiple - random
    q = np.random.uniform(size=(3,128))
    p = np.random.uniform(size=(3,128))
    L = CartesianPhaseSpacePosition(q, p).angular_momentum()
    assert L.shape == (3,128)

def test_combine():

    o1 = CartesianPhaseSpacePosition.from_w(np.random.random(size=6), units=galactic)
    o2 = CartesianPhaseSpacePosition.from_w(np.random.random(size=6), units=galactic)
    o = combine((o1, o2))
    assert o.pos.shape == (3,2)
    assert o.vel.shape == (3,2)

    o1 = CartesianPhaseSpacePosition.from_w(np.random.random(size=6), units=galactic)
    o2 = CartesianPhaseSpacePosition.from_w(np.random.random(size=(6,10)), units=galactic)
    o = combine((o1, o2))
    assert o.pos.shape == (3,11)
    assert o.vel.shape == (3,11)

    o1 = CartesianPhaseSpacePosition.from_w(np.random.random(size=6), units=galactic)
    o2 = CartesianPhaseSpacePosition.from_w(np.random.random(size=6), units=solarsystem)
    o = combine((o1, o2))
    assert o.pos.unit == galactic['length']
    assert o.vel.unit == galactic['length']/galactic['time']



# coding: utf-8

from __future__ import division, print_function

__author__ = "adrn <adrn@astro.columbia.edu>"

# Third-party
import astropy.units as u
import numpy as np
from scipy.signal import argrelmin

# Project
from . import CartesianPhaseSpacePosition, CartesianOrbit

__all__ = ['fast_lyapunov_max', 'lyapunov_max', 'surface_of_section']

def fast_lyapunov_max(w0, potential, dt, n_steps, d0=1e-5,
                      n_steps_per_pullback=10, noffset_orbits=2, t1=0.,
                      atol=1E-10, rtol=1E-10, nmax=0, return_orbit=True):
    """
    Compute the maximum Lyapunov exponent using a C-implemented estimator
    that uses the DOPRI853 integrator.

    Parameters
    ----------
    w0 : `~gala.dynamics.PhaseSpacePosition`, array_like
        Initial conditions.
    dt : numeric
        Timestep.
    n_steps : int
        Number of steps to run for.
    d0 : numeric (optional)
        The initial separation.
    n_steps_per_pullback : int (optional)
        Number of steps to run before re-normalizing the offset vectors.
    noffset_orbits : int (optional)
        Number of offset orbits to run.
    t1 : numeric (optional)
        Time of initial conditions. Assumed to be t=0.
    return_orbit : bool (optional)
        Store the full orbit for the parent and all offset orbits.

    Returns
    -------
    LEs : :class:`~astropy.units.Quantity`
        Lyapunov exponents calculated from each offset / deviation orbit.
    orbit : `~gala.dynamics.CartesianOrbit` (optional)

    """

    from .lyapunov import dop853_lyapunov_max, dop853_lyapunov_max_dont_save

    if not hasattr(potential, 'c_instance'):
        raise TypeError("Input potential must be a CPotential subclass.")

    if not isinstance(w0, CartesianPhaseSpacePosition):
        w0 = np.asarray(w0)
        ndim = w0.shape[0]//2
        w0 = CartesianPhaseSpacePosition(pos=w0[:ndim],
                                         vel=w0[ndim:])

    _w0 = np.squeeze(w0.w(potential.units))
    if _w0.ndim > 1:
        raise ValueError("Can only compute fast Lyapunov exponent for a single orbit.")

    if return_orbit:
        t,w,l = dop853_lyapunov_max(potential.c_instance, _w0,
                                    dt, n_steps+1, t1,
                                    d0, n_steps_per_pullback, noffset_orbits,
                                    atol, rtol, nmax)
        w = np.rollaxis(w, -1)

        try:
            tunit = potential.units['time']
        except (TypeError, AttributeError):
            tunit = u.dimensionless_unscaled

        orbit = CartesianOrbit.from_w(w=w, units=potential.units, t=t*tunit, potential=potential)
        return l/tunit, orbit
    else:
        l = dop853_lyapunov_max_dont_save(potential.c_instance, _w0,
                                          dt, n_steps+1, t1,
                                          d0, n_steps_per_pullback, noffset_orbits,
                                          atol, rtol, nmax)

        try:
            tunit = potential.units['time']
        except (TypeError, AttributeError):
            tunit = u.dimensionless_unscaled

        return l/tunit

def lyapunov_max(w0, integrator, dt, n_steps, d0=1e-5, n_steps_per_pullback=10,
                 noffset_orbits=8, t1=0., units=None):
    """

    Compute the maximum Lyapunov exponent of an orbit by integrating many
    nearby orbits (``noffset``) separated with isotropically distributed
    directions but the same initial deviation length, ``d0``. This algorithm
    re-normalizes the offset orbits every ``n_steps_per_pullback`` steps.

    Parameters
    ----------
    w0 : `~gala.dynamics.PhaseSpacePosition`, array_like
        Initial conditions.
    integrator : `~gala.integrate.Integrator`
        An instantiated `~gala.integrate.Integrator` object. Must have a run() method.
    dt : numeric
        Timestep.
    n_steps : int
        Number of steps to run for.
    d0 : numeric (optional)
        The initial separation.
    n_steps_per_pullback : int (optional)
        Number of steps to run before re-normalizing the offset vectors.
    noffset_orbits : int (optional)
        Number of offset orbits to run.
    t1 : numeric (optional)
        Time of initial conditions. Assumed to be t=0.
    units : `~gala.units.UnitSystem` (optional)
        If passing in an array (not a `~gala.dynamics.PhaseSpacePosition`),
        you must specify a unit system.

    Returns
    -------
    LEs : :class:`~astropy.units.Quantity`
        Lyapunov exponents calculated from each offset / deviation orbit.
    orbit : `~gala.dynamics.CartesianOrbit`
    """

    if units is not None:
        pos_unit = units['length']
        vel_unit = units['length']/units['time']
    else:
        pos_unit = u.dimensionless_unscaled
        vel_unit = u.dimensionless_unscaled

    if not isinstance(w0, CartesianPhaseSpacePosition):
        w0 = np.asarray(w0)
        ndim = w0.shape[0]//2
        w0 = CartesianPhaseSpacePosition(pos=w0[:ndim]*pos_unit,
                                         vel=w0[ndim:]*vel_unit)

    _w0 = w0.w(units)
    ndim = 2*w0.ndim

    # number of iterations
    niter = n_steps // n_steps_per_pullback

    # define offset vectors to start the offset orbits on
    d0_vec = np.random.uniform(size=(ndim,noffset_orbits))
    d0_vec /= np.linalg.norm(d0_vec, axis=0)[np.newaxis]
    d0_vec *= d0

    w_offset = _w0 + d0_vec
    all_w0 = np.hstack((_w0,w_offset))

    # array to store the full, main orbit
    full_w = np.zeros((ndim,n_steps+1,noffset_orbits+1))
    full_w[:,0] = all_w0
    full_ts = np.zeros((n_steps+1,))
    full_ts[0] = t1

    # arrays to store the Lyapunov exponents and times
    LEs = np.zeros((niter,noffset_orbits))
    ts = np.zeros_like(LEs)
    time = t1
    for i in range(1,niter+1):
        ii = i * n_steps_per_pullback

        orbit = integrator.run(all_w0, dt=dt, n_steps=n_steps_per_pullback, t1=time)
        tt = orbit.t.value
        ww = orbit.w(units)
        time += dt*n_steps_per_pullback

        main_w = ww[:,-1,0:1]
        d1 = ww[:,-1,1:] - main_w
        d1_mag = np.linalg.norm(d1, axis=0)

        LEs[i-1] = np.log(d1_mag/d0)
        ts[i-1] = time

        w_offset = ww[:,-1,0:1] + d0 * d1 / d1_mag[np.newaxis]
        all_w0 = np.hstack((ww[:,-1,0:1],w_offset))

        full_w[:,(i-1)*n_steps_per_pullback+1:ii+1] = ww[:,1:]
        full_ts[(i-1)*n_steps_per_pullback+1:ii+1] = tt[1:]

    LEs = np.array([LEs[:ii].sum(axis=0)/ts[ii-1] for ii in range(1,niter)])

    try:
        t_unit = units['time']
    except (TypeError, AttributeError):
        t_unit = u.dimensionless_unscaled

    orbit = CartesianOrbit.from_w(w=full_w, units=units, t=full_ts*t_unit)
    return LEs/t_unit, orbit

def surface_of_section(orbit, plane_ix, interpolate=False):
    """
    Generate and return a surface of section from the given orbit.

    .. warning::

        This is an experimental function and the API may change.

    Parameters
    ----------
    orbit : `~gala.dynamics.CartesianOrbit`
    plane_ix : int
        Integer that represents the coordinate to record crossings in. For
        example, for a 2D Hamiltonian where you want to make a SoS in
        :math:`y-p_y`, you would specify ``plane_ix=0`` (crossing the
        :math:`x` axis), and this will only record crossings for which
        :math:`p_x>0`.
    interpolate : bool (optional)
        Whether or not to interpolate on to the plane of interest. This
        makes it much slower, but will work for orbits with a coarser
        sampling.

    Returns
    -------

    Examples
    --------
    If your orbit of interest is a tube orbit, it probably conserves (at
    least approximately) some equivalent to angular momentum in the direction
    of the circulation axis. Therefore, a surface of section in R-z should
    be instructive for classifying these orbits. TODO...show how to convert
    an orbit to Cylindrical..etc...

    """

    w = orbit.w()
    if w.ndim == 2:
        w = w[...,None]

    ndim,ntimes,norbits = w.shape
    H_dim = ndim // 2
    p_ix = plane_ix + H_dim

    if interpolate:
        raise NotImplementedError("Not yet implemented, sorry!")

    # record position on specified plane when orbit crosses
    all_sos = np.zeros((ndim,norbits), dtype=object)
    for n in range(norbits):
        cross_ix = argrelmin(w[plane_ix,:,n]**2)[0]
        cross_ix = cross_ix[w[p_ix,cross_ix,n] > 0.]
        sos = w[:,cross_ix,n]

        for j in range(ndim):
            all_sos[j,n] = sos[j,:]

    return all_sos



# coding: utf-8

""" Test dynamics plotting  """

from __future__ import division, print_function

__author__ = "adrn <adrn@astro.columbia.edu>"

# Standard library
import os
import logging

# Third-party
import numpy as np
from astropy import log as logger
import gala.dynamics as gd

# Project
from ..plot import *

logger.setLevel(logging.DEBUG)

def test_orbits(tmpdir):

    # generate an "orbit"
    n = 8
    t = np.linspace(0, 100, 1000).reshape(1000,1)
    x = np.cos(np.random.uniform(1.,8.,size=(1,n))*t).T[None]
    y = np.sin(np.random.uniform(1.,8.,size=(1,n))*t).T[None]
    z = np.cos(np.random.uniform(1.,8.,size=(1,n))*t).T[None]
    vx = vy = vz = np.zeros_like(x)
    w = np.rollaxis(np.vstack((x,y,z,vx,vy,vz)), -1, 1)

    fig = plot_orbits(w, linestyle='none', marker='.', alpha=0.25)
    fig.savefig(os.path.join(str(tmpdir), "all_orbits.png"))

    fig = plot_orbits(w, ix=0, linestyle='none', marker='.', alpha=0.25)
    fig.savefig(os.path.join(str(tmpdir), "one_orbit.png"))

    fig = plot_orbits(w, ix=0, linestyle='none', marker='.', alpha=0.25,
                      labels=("herp","derp","merp"))
    fig.savefig(os.path.join(str(tmpdir), "one_orbit_labels.png"))

    fig = plot_orbits(w, triangle=True, linestyle='-', marker=None)
    fig.savefig(os.path.join(str(tmpdir), "all_orbits_triangle.png"))

    fig = plot_orbits(w, ix=0, triangle=True, linestyle='-', marker=None)
    fig.savefig(os.path.join(str(tmpdir), "one_orbit_triangle.png"))

    fig = plot_orbits(w, ix=0, triangle=True, linestyle='-', marker=None,
                      labels=("herp","derp","merp"))
    fig.savefig(os.path.join(str(tmpdir), "one_orbit_triangle_labels.png"))

def test_three_panel(tmpdir):

    q = np.random.uniform(0.,10.,size=(1000,3))
    q0 = np.array([5,5,5])

    fig = three_panel(q)
    fig.savefig(os.path.join(str(tmpdir), "three-panel-random.png"))

    fig = three_panel(q, triangle=True)
    fig.savefig(os.path.join(str(tmpdir), "three-panel-random_triangle.png"))

    fig = three_panel(q, relative_to=q0, labels=[r'$\Omega_1$',r'$\Omega_2$',r'$\Omega_3$'])
    fig.savefig(os.path.join(str(tmpdir), "three-panel-random-relative.png"))

    fig = three_panel(q, relative_to=q0, triangle=True, labels=[r'$\Omega_1$',r'$\Omega_2$',r'$\Omega_3$'])
    fig.savefig(os.path.join(str(tmpdir), "three-panel-random-relative_triangle.png"))

def test_1d(tmpdir):

    t = np.linspace(0,100.,1000)
    q = np.cos(2*np.pi*t/10.)[None]

    fig = plot_orbits(q, labels=(r"$\theta$",))
    fig.savefig(os.path.join(str(tmpdir), "1d-orbit-labels.png"))

    fig = plot_orbits(q, t=t, labels=(r"$\theta$",))
    fig.savefig(os.path.join(str(tmpdir), "1d-orbit-labels-time.png"))

def test_2d(tmpdir):

    t = np.linspace(0,100.,1000)

    q = np.zeros((2,len(t)))
    q[0,:] = np.cos(2*np.pi*t/10.)
    q[1,:] = np.sin(2*np.pi*t/5.5)

    fig = plot_orbits(q, labels=(r"$\theta$",r"$\omega$"))
    fig.savefig(os.path.join(str(tmpdir), "2d-orbit-labels.png"))



# coding: utf-8
"""
    Test the coordinates class that represents the plane of orbit of the Sgr dwarf galaxy.
"""

from __future__ import absolute_import, unicode_literals, division, print_function

__author__ = "adrn <adrn@astro.columbia.edu>"

# Third-party
import astropy.coordinates as coord
import astropy.units as u
from astropy.utils.data import get_pkg_data_filename
import numpy as np

# This package
from ..sgr import *

def test_simple():
    c = coord.ICRS(coord.Angle(217.2141, u.degree),
                   coord.Angle(-11.4351, u.degree))
    c.transform_to(Sagittarius)

    c = coord.Galactic(coord.Angle(217.2141, u.degree),
                       coord.Angle(-11.4351, u.degree))
    c.transform_to(Sagittarius)

    c = Sagittarius(coord.Angle(217.2141, u.degree),
                    coord.Angle(-11.4351, u.degree))
    c.transform_to(coord.ICRS)
    c.transform_to(coord.Galactic)

    c = coord.Galactic(coord.Angle(217.2141, u.degree),
                       coord.Angle(-11.4351, u.degree))
    s = c.transform_to(Sagittarius)

    # with distance
    c = Sagittarius(coord.Angle(217.2141, u.degree),
                    coord.Angle(-11.4351, u.degree),
                    distance=15*u.kpc)
    c.transform_to(coord.ICRS)
    c2 = c.transform_to(coord.Galactic)
    assert np.allclose(c2.distance.value, c.distance.value)

def test_against_David_Law():
    """ Test my code against an output file from using David Law's cpp code. Do:

            g++ SgrCoord.cpp; ./a.out

        to generate the data file, SgrCoord_data.

    """
    filename = get_pkg_data_filename('SgrCoord_data')
    law_data = np.genfromtxt(filename, names=True, delimiter=',')

    c = coord.Galactic(law_data["l"]*u.deg, law_data["b"]*u.deg)
    sgr_coords = c.transform_to(Sagittarius)

    law_sgr_coords = Sagittarius(Lambda=law_data["lambda"]*u.deg, Beta=law_data["beta"]*u.deg)

    sep = sgr_coords.separation(law_sgr_coords).arcsec*u.arcsec
    assert np.all(sep < 1.*u.arcsec)



# coding: utf-8
"""
    Test the builtin CPotential classes
"""

from __future__ import absolute_import, unicode_literals, division, print_function

__author__ = "adrn <adrn@astro.columbia.edu>"

# Third-party
import numpy as np
import astropy.units as u
import pytest

# This project
from ..core import CompositePotential
from ..builtin import *
from ...units import solarsystem, galactic, DimensionlessUnitSystem
from .helpers import PotentialTestBase, CompositePotentialTestBase

##############################################################################
# Python
##############################################################################

class TestHarmonicOscillator1D(PotentialTestBase):
    potential = HarmonicOscillatorPotential(omega=1.)
    w0 = [1.,0.]

    def test_plot(self):
        # Skip for now because contour plotting assumes 3D
        pass

class TestHarmonicOscillator2D(PotentialTestBase):
    potential = HarmonicOscillatorPotential(omega=[1.,2])
    w0 = [1.,0.5,0.,0.1]

    def test_plot(self):
        # Skip for now because contour plotting assumes 3D
        pass

##############################################################################
# Cython
##############################################################################

class TestHenonHeiles(PotentialTestBase):
    potential = HenonHeilesPotential()
    w0 = [1.,0.,0.,0.,2*np.pi,0.]

class TestKepler(PotentialTestBase):
    potential = KeplerPotential(units=solarsystem, m=1.)
    w0 = [1.,0.,0.,0.,2*np.pi,0.]

class TestKeplerUnitInput(PotentialTestBase):
    potential = KeplerPotential(units=solarsystem, m=1E-3*u.ksolMass)
    w0 = [1.,0.,0.,0.,2*np.pi,0.]

class TestIsochrone(PotentialTestBase):
    potential = IsochronePotential(units=solarsystem, m=1., b=0.1)
    w0 = [1.,0.,0.,0.,2*np.pi,0.]

class TestIsochroneDimensionless(PotentialTestBase):
    potential = IsochronePotential(units=DimensionlessUnitSystem(), m=1., b=0.1)
    w0 = [1.,0.,0.,0.,2*np.pi,0.]

class TestHernquist(PotentialTestBase):
    potential = HernquistPotential(units=galactic, m=1.E11, c=0.26)
    w0 = [1.,0.,0.,0.,0.1,0.1]

class TestPlummer(PotentialTestBase):
    potential = PlummerPotential(units=galactic, m=1.E11, b=0.26)
    w0 = [1.,0.,0.,0.,0.1,0.1]

class TestJaffe(PotentialTestBase):
    potential = JaffePotential(units=galactic, m=1.E11, c=0.26)
    w0 = [1.,0.,0.,0.,0.1,0.1]

class TestMiyamotoNagai(PotentialTestBase):
    potential = MiyamotoNagaiPotential(units=galactic, m=1.E11, a=6.5, b=0.26)
    w0 = [8.,0.,0.,0.,0.22,0.1]

class TestStone(PotentialTestBase):
    potential = StonePotential(units=galactic, m=1E11, r_c=0.1, r_h=10.)
    w0 = [8.,0.,0.,0.,0.18,0.1]

class TestSphericalNFWPotential(PotentialTestBase):
    potential = SphericalNFWPotential(units=galactic, v_c=0.35, r_s=12.)
    w0 = [19.0,2.7,-6.9,0.0352238,-0.03579493,0.075]

    def test_against_triaxial(self):
        other = LeeSutoTriaxialNFWPotential(units=galactic,
                                            v_c=0.35, r_s=12.,
                                            a=1., b=1., c=1.)

        v1 = other.value(self.w0[:3])
        v2 = self.potential.value(self.w0[:3])
        assert np.allclose(v1.value,v2.value)

        a1 = other.gradient(self.w0[:3])
        a2 = self.potential.gradient(self.w0[:3])
        assert np.allclose(a1.value,a2.value)

    def test_mass_enclosed(self):

        # true mass profile
        vc = self.potential.parameters['v_c'].value
        rs = self.potential.parameters['r_s'].value
        G = self.potential.G

        r = np.linspace(1., 400, 100)
        fac = np.log(1 + r/rs) - (r/rs) / (1 + (r/rs))
        true_mprof = vc**2*rs / (np.log(2)-0.5) / G * fac

        R = np.zeros((3,len(r)))
        R[0,:] = r
        esti_mprof = self.potential.mass_enclosed(R)

        assert np.allclose(true_mprof, esti_mprof.value, rtol=1E-6)

class TestFlattenedNFW(PotentialTestBase):
    potential = FlattenedNFWPotential(units=galactic, v_c=0.2, r_s=12., q_z=0.9)
    w0 = [0.,20.,0.,0.0352238,-0.03579493,0.175]

class TestLeeSutoTriaxialNFW(PotentialTestBase):
    potential = LeeSutoTriaxialNFWPotential(units=galactic, v_c=0.35, r_s=12.,
                                            a=1.3, b=1., c=0.8)
    w0 = [19.0,2.7,-6.9,0.0352238,-0.03579493,0.075]

class TestLogarithmic(PotentialTestBase):
    potential = LogarithmicPotential(units=galactic, v_c=0.17, r_h=10.,
                                     q1=1.2, q2=1., q3=0.8)
    w0 = [19.0,2.7,-6.9,0.0352238,-0.03579493,0.075]

class TestComposite(CompositePotentialTestBase):
    p1 = LogarithmicPotential(units=galactic,
                              v_c=0.17, r_h=10.,
                              q1=1.2, q2=1., q3=0.8)
    p2 = MiyamotoNagaiPotential(units=galactic,
                                m=1.E11, a=6.5, b=0.26)
    potential = CompositePotential()
    potential['disk'] = p2
    potential['halo'] = p1
    w0 = [19.0,2.7,-6.9,0.0352238,-0.03579493,0.075]



# coding: utf-8

from __future__ import division, print_function

__author__ = "adrn <adrn@astro.columbia.edu>"

# Standard library
from collections import OrderedDict

# Third-party
import numpy as np

from ..core import PotentialBase
from ...util import atleast_2d

__all__ = ["HarmonicOscillatorPotential", "KuzminPotential"]

class HarmonicOscillatorPotential(PotentialBase):
    r"""
    Represents an N-dimensional harmonic oscillator.

    .. math::

        \Phi = \frac{1}{2}\omega^2 x^2

    Parameters
    ----------
    omega : numeric
        Frequency.
    units : iterable(optional)
        Unique list of non-reducable units that specify (at minimum) the
        length, mass, time, and angle units.
    """

    def __init__(self, omega, units=None):
        parameters = OrderedDict()
        parameters['omega'] = np.array(omega)
        super(HarmonicOscillatorPotential, self).__init__(units=units,
                                                          parameters=parameters)

    def _value(self, x, t):
        om = np.array(self.parameters['omega'])
        return np.sum((0.5 * om**2 * x.T**2).T, axis=0)

    def _gradient(self, x, t):
        om = np.array(self.parameters['omega'])
        return (om**2*x.T).T

    def action_angle(self, w):
        """
        Transform the input cartesian position and velocity to action-angle
        coordinates the Harmonic Oscillator potential. This transformation
        is analytic and can be used as a "toy potential" in the
        Sanders & Binney 2014 formalism for computing action-angle coordinates
        in _any_ potential.

        Adapted from Jason Sanders' code
        `genfunc <https://github.com/jlsanders/genfunc>`_.

        Parameters
        ----------
        w : :class:`gala.dynamics.CartesianPhaseSpacePosition`, :class:`gala.dynamics.CartesianOrbit`
            The positions or orbit to compute the actions, angles, and frequencies at.
        """
        from ...dynamics.analyticactionangle import harmonic_oscillator_to_aa
        return harmonic_oscillator_to_aa(w, self)

    # def phase_space(self, actions, angles):
    #     """
    #     Transform the input action-angle coordinates to cartesian position and velocity
    #     assuming a Harmonic Oscillator potential. This transformation
    #     is analytic and can be used as a "toy potential" in the
    #     Sanders & Binney 2014 formalism for computing action-angle coordinates
    #     in _any_ potential.

    #     Adapted from Jason Sanders' code
    #     `genfunc <https://github.com/jlsanders/genfunc>`_.

    #     Parameters
    #     ----------
    #     x : array_like
    #         Positions.
    #     v : array_like
    #         Velocities.
    #     """
    #     from ...dynamics.analyticactionangle import harmonic_oscillator_aa_to_xv
    #     return harmonic_oscillator_aa_to_xv(actions, angles, self)

class KuzminPotential(PotentialBase):
    r"""
    The Kuzmin flattened disk potential.

    .. math::

        \Phi = -\frac{Gm}{\sqrt{x^2 + y^2 + (a + |z|)^2}}

    Parameters
    ----------
    m : numeric
        Mass.
    a : numeric
        Flattening parameter.
    units : iterable
        Unique list of non-reducable units that specify (at minimum) the
        length, mass, time, and angle units.

    """
    def __init__(self, m, a, units):
        parameters = OrderedDict()
        parameters['m'] = m
        parameters['a'] = a
        super(KuzminPotential, self).__init__(units=units,
                                              parameters=parameters)

    def _value(self, q, t):
        x,y,z = q
        m = self.parameters['m']
        a = self.parameters['a']
        val = -self.G * m / np.sqrt(x**2 + y**2 + (a + np.abs(z))**2)
        return val

    def _gradient(self, q, t):
        x,y,z = q
        m = self.parameters['m']
        a = self.parameters['a']
        fac = self.G * m / (x**2 + y**2 + (a + np.abs(z))**2)**1.5
        return fac[None,...] * q



# coding: utf-8

""" Convenience functions for observational data """

from __future__ import division, print_function

__author__ = "adrn <adrn@astro.columbia.edu>"

# Third-party
import numpy as np
import astropy.units as u

__all__ = ["apparent_magnitude", "absolute_magnitude",
           "distance_modulus", "distance"]

def apparent_magnitude(mag_abs, distance):
    """
    Compute the apparent magnitude of a source given an absolute magnitude
    and a distance.

    Parameters
    ----------
    mag_abs : numeric or iterable
        Absolute magnitude.
    distance : :class:`~astropy.units.Quantity`
        The distance to the source as a Quantity object.

    Returns
    -------
    mag_app : :class:`~numpy.ndarray`
        The apparent magnitude.
    """

    # Compute the apparent magnitude
    return mag_abs + distance_modulus(distance)

def absolute_magnitude(mag_app, distance):
    """
    Compute the absolute magnitude of a source given an apparent magnitude
    and a distance.

    Parameters
    ----------
    mag_app : numeric or iterable
        Apparent magnitude.
    distance : :class:`~astropy.units.Quantity`
        The distance to the source as a Quantity object.

    Returns
    -------
    mag_abs : :class:`~numpy.ndarray`
        The absolute magnitude.
    """

    # Compute the absolute magnitude
    return mag_app - distance_modulus(distance)

def distance_modulus(distance):
    """
    Compute the distance modulus given a distance.

    Parameters
    ----------
    distance : astropy.units.Quantity
        The distance as a Quantity object.

    Returns
    -------
    distance_mod : :class:`~numpy.ndarray`
        The distance modulus.
    """

    return 5.*(np.log10(distance.to(u.pc).value) - 1.)

def distance(distance_mod):
    """
    Compute the distance from the distance modulus.

    Parameters
    ----------
    distance_mod : astropy.units.Quantity
        The distance modulus.

    Returns
    -------
    distance : :class:`~astropy.units.Quantity`
        Distance.

    """
    return 10**(distance_mod/5. + 1) * u.pc



# coding: utf-8
"""
    Test the Cython integrators.
"""

from __future__ import absolute_import, unicode_literals, division, print_function

__author__ = "adrn <adrn@astro.columbia.edu>"

# Standard library
import os
import time

# Third-party
import numpy as np
import matplotlib.pyplot as pl
import pytest

# Project
from ..pyintegrators.leapfrog import LeapfrogIntegrator
from ..cyintegrators.leapfrog import leapfrog_integrate_potential
from ..pyintegrators.dopri853 import DOPRI853Integrator
from ..cyintegrators.dop853 import dop853_integrate_potential
from ...potential import HernquistPotential
from ...units import galactic

integrator_list = [LeapfrogIntegrator, DOPRI853Integrator]
func_list = [leapfrog_integrate_potential, dop853_integrate_potential]
_list = zip(integrator_list, func_list)

# ----------------------------------------------------------------------------

@pytest.mark.parametrize(("Integrator","integrate_func"), _list)
def test_compare_to_py(Integrator, integrate_func):
    p = HernquistPotential(m=1E11, c=0.5, units=galactic)

    def F(t,w):
        dq = w[3:]
        dp = -p._gradient(w[:3])
        return np.vstack((dq,dp))

    cy_w0 = np.array([[0.,10.,0.,0.2,0.,0.],
                      [10.,0.,0.,0.,0.2,0.]])
    py_w0 = np.ascontiguousarray(cy_w0.T)

    n_steps = 10000
    dt = 2.
    t = np.linspace(0,dt*n_steps,n_steps+1)

    cy_t,cy_w = integrate_func(p.c_instance, cy_w0, t)
    cy_w = np.rollaxis(cy_w, -1)

    integrator = Integrator(F)
    orbit = integrator.run(py_w0, dt=dt, n_steps=n_steps)
    py_t = orbit.t.value
    py_w = orbit.w()

    assert py_w.shape == cy_w.shape
    assert np.allclose(cy_w[:,-1], py_w[:,-1])

@pytest.mark.skipif(True, reason="For timing locally")
def test_time_integration():
    niter = 100

    p = HernquistPotential(m=1E11, c=0.5, units=galactic)

    cy_w0 = np.array([[0.,10.,0.,0.2,0.,0.],
                      [10.,0.,0.,0.,0.2,0.]])

    n_steps = 10000
    dt = 2.
    t = np.linspace(0,dt*n_steps,n_steps+1)

    time0 = time.time()
    for i in range(niter):
        t,w = dop853_integrate_potential(p.c_instance, cy_w0, t)

    exec_time = (time.time()-time0) / niter
    print("{:.3f} per loop iteration".format(exec_time))

@pytest.mark.skipif(True, reason="Slow test - mainly for plotting locally")
@pytest.mark.parametrize(("Integrator","integrate_func"), _list)
def test_scaling(tmpdir, Integrator, integrate_func):
    p = HernquistPotential(m=1E11, c=0.5, units=galactic)

    def F(t,w):
        dq = w[3:]
        dp = -p._gradient(w[:3])
        return np.vstack((dq,dp))

    step_bins = np.logspace(2,np.log10(25000),7)
    colors = ['k', 'b', 'r']
    dt = 1.

    for c,nparticles in zip(colors,[1, 100, 1000]):
        cy_w0 = np.array([[0.,10.,0.,0.2,0.,0.]]*nparticles)
        py_w0 = np.ascontiguousarray(cy_w0.T)

        x = []
        cy_times = []
        py_times = []
        for n_steps in step_bins:
            print(nparticles, n_steps)
            t = np.linspace(0,dt*n_steps,n_steps+1)
            x.append(n_steps)

            # time the Cython integration
            t0 = time.time()
            integrate_func(p.c_instance, cy_w0, t)
            cy_times.append(time.time() - t0)

            # time the Python integration
            t0 = time.time()
            integrator = Integrator(F)
            orbit = integrator.run(py_w0, dt=dt, n_steps=n_steps)
            py_times.append(time.time() - t0)

    #     pl.loglog(x, cy_times, linestyle='-', lw=2., c=c, marker=None,
    #               label="cy: {} orbits".format(nparticles))
    #     pl.loglog(x, py_times, linestyle='--', lw=2., c=c, marker=None,
    #               label="py: {} orbits".format(nparticles))

    # pl.title(Integrator.__name__)
    # pl.legend(loc='upper left')
    # pl.xlim(90,30000)
    # pl.xlabel("N steps")
    # pl.tight_layout()
    # # pl.show()
    # pl.savefig(os.path.join(tmpdir, "integrate-scaling.png"), dpi=300)
